Package: coarseDataTools
Version: 0.7.2
Date: 2025-04-09
Title: Analysis of Coarsely Observed Data
Authors@R: c(person("Nicholas G.", "Reich", email = "nick@umass.edu", role = c("aut", "cre")),
  person("Justin", "Lessler", email = "jlessler@unc.edu", role = "aut"),
  person("Andrew", "Azman", email = "andrewazman@gmail.com", role = "aut"),
  person("Zhian N.", "Kamvar", email = "zkamvar@gmail.com", role = "ctb"),
  person("Hugo", "Gruson", email = "hugo.gruson@normalesup.org", role = "ctb"))
Maintainer: Nicholas G. Reich <nick@umass.edu>
License: GPL (>= 2)
URL: http://nickreich.github.io/coarseDataTools/
Encoding: UTF-8
Depends: R (>= 3.5.0)
Imports: MCMCpack, graphics, stats, methods
Suggests: markdown, knitr
Description: Functions to analyze coarse data.
    Specifically, it contains functions to (1) fit parametric accelerated
    failure time models to interval-censored survival time data, and (2)
    estimate the case-fatality ratio in scenarios with under-reporting.
    This package's development was motivated by applications to infectious
    disease: in particular, problems with estimating the incubation period and
    the case fatality ratio of a given disease.  Sample data files are included
    in the package. See Reich et al. (2009) <doi:10.1002/sim.3659>, 
    Reich et al. (2012) <doi:10.1111/j.1541-0420.2011.01709.x>, and 
    Lessler et al. (2009) <doi:10.1016/S1473-3099(09)70069-6>.
Collate: 'CFR_estimation.R' 'dic.fit.mcmc.R' 'dic.fit.R'
        'get.obs.type.R' 'S4stuff.R' 'sampleSizeSimulation.R'
        'coarseDataTools-package.R'
VignetteBuilder: knitr
RoxygenNote: 7.3.2
NeedsCompilation: no
Packaged: 2025-04-11 20:15:33 UTC; nick
Author: Nicholas G. Reich [aut, cre],
  Justin Lessler [aut],
  Andrew Azman [aut],
  Zhian N. Kamvar [ctb],
  Hugo Gruson [ctb]
Repository: CRAN
Date/Publication: 2025-04-11 21:00:02 UTC

# Generated by roxygen2: do not edit by hand

export(EMforCFR)
export(dic.fit)
export(dic.fit.mcmc)
export(generate.coarse.data)
export(get.obs.type)
export(loglikhd)
export(precision.simulation)
export(precision.simulation.coarse)
export(precision.simulation.exact)
exportClasses(cd.fit)
exportClasses(cd.fit.mcmc)
exportMethods(logLik)
importFrom(MCMCpack,MCMCmetrop1R)
importFrom(grDevices,rgb)
importFrom(graphics,lines)
importFrom(methods,is)
importFrom(methods,new)
importFrom(stats,coef)
importFrom(stats,dgamma)
importFrom(stats,dlnorm)
importFrom(stats,dnbinom)
importFrom(stats,dnorm)
importFrom(stats,dweibull)
importFrom(stats,glm)
importFrom(stats,integrate)
importFrom(stats,logLik)
importFrom(stats,optim)
importFrom(stats,optimize)
importFrom(stats,pgamma)
importFrom(stats,plnorm)
importFrom(stats,poisson)
importFrom(stats,pweibull)
importFrom(stats,qgamma)
importFrom(stats,qlnorm)
importFrom(stats,qnorm)
importFrom(stats,qt)
importFrom(stats,quantile)
importFrom(stats,qweibull)
importFrom(stats,rlnorm)
importFrom(stats,rnorm)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,vcov)
importFrom(utils,data)
importFrom(utils,setTxtProgressBar)
importFrom(utils,txtProgressBar)

coarseDataTools
===============

<!-- badges: start -->
[![CRAN](http://cranlogs.r-pkg.org/badges/coarseDataTools)](https://cran.r-project.org/package=coarseDataTools)
[![R-CMD-check](https://github.com/nickreich/coarseDataTools/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/nickreich/coarseDataTools/actions/workflows/R-CMD-check.yaml)
<!-- badges: end -->

This is the repository for the coarseDataTools R package. We use this as a development space. The most recent, stable version of the package can be downloaded either from CRAN (https://cran.r-project.org/package=coarseDataTools) or from the most recent release version on github (https://github.com/nickreich/coarseDataTools/releases).

This package contains functions to analyze coarsely observed data.
    Specifically, it contains functions to (1) fit parametric accelerated
    failure time models to interval-censored survival time data, and (2)
    estimate the case-fatality ratio in scenarios with underreporting.
    This package's development was motivated by applications to infectious
    disease: in particular, problems with estimating the incubation period and
    the case fatality ratio of a given disease. Sample data files are included
    in the package.


As of March 2016, coarseDataTools imports functions from the MCMCpack package, which in turn imports functions from the graph and Rgraphviz packages. Both of these packages have been removed from CRAN, but can be installed from Bioconductor using the following code:

```
source("https://bioconductor.org/biocLite.R")
biocLite("graph")
biocLite("Rgraphviz")
```

## ----label=sourcing-----------------------------------------------------------
library(coarseDataTools)
data(simulated.outbreak.deaths)

## ----label=datapeek-----------------------------------------------------------
simulated.outbreak.deaths[15:20, ]

## ----label=preprocessing------------------------------------------------------
## set minimum number of observed cases for inclusion
min.cases <- 10

## observed cases
N.1 <- simulated.outbreak.deaths[1:60, "N"]
N.2 <- simulated.outbreak.deaths[61:120, "N"]

## subset to run analyis on times with greater than min.cases
first.t <- min(which(N.1 > min.cases & N.2 > min.cases))
last.t <- max(which(N.1 > min.cases & N.2 > min.cases))
idx.for.Estep <- first.t:last.t

## find and label the subset of times to be used for estimation routine
new.times <- seq_along(idx.for.Estep)
simulated.outbreak.deaths <- cbind(simulated.outbreak.deaths, new.times = NA)
simulated.outbreak.deaths[c(idx.for.Estep, idx.for.Estep + 60), "new.times"] <- rep(new.times, 2)

## ----label=datapeek2----------------------------------------------------------
simulated.outbreak.deaths[15:20, ]

## ----label=setValues----------------------------------------------------------
assumed.nu <- c(0, 0.3, 0.4, 0.3)
alpha.start <- rep(0, 22)

## ----label=runAnalysis, cache=TRUE, warning=FALSE-----------------------------
cfr.ests <- EMforCFR(
  assumed.nu = assumed.nu,
  alpha.start.values = alpha.start, full.data = simulated.outbreak.deaths, verb = FALSE,
  SEM.var = TRUE, max.iter = 100, tol = 1e-5
)

## ----label=estimationResults--------------------------------------------------
cfr.ests$naive.rel.cfr
cfr.ests$glm.rel.cfr
cfr.ests$EM.rel.cfr
cfr.ests$EM.rel.cfr.var.SEM

##' @importFrom MCMCpack MCMCmetrop1R
##' @importFrom stats logLik
##' @importFrom grDevices  rgb
##' @importFrom graphics  lines
##' @importFrom methods  new
##' @importFrom stats  coef  dgamma  dlnorm  dnbinom  dnorm  dweibull  glm  integrate  optim  optimize  pgamma  plnorm  poisson  pweibull  qgamma  qlnorm  qnorm  qt  quantile  qweibull  rlnorm  rnorm  runif  sd  vcov
##' @importFrom utils  data  setTxtProgressBar  txtProgressBar

##' @name exp.win.lengths
##' @title Exposure window lengths from an influenza outbreak at a NYC school
##' @description A numeric vector of exposure window lengths taken from a dataset of doubly interval-censored incubation period observations.  All observations came from a NYC public school.  The outbreak has been described in full in Lessler et al. (see citation below).
##' @docType data
##' @format A numeric vector with 134 positive values.  Each value represents an exposure window length from an observation of the incubation period for that individual.  The exposure window length is the length of time during which exposure could have occurred.  For example, if an individual could have been exposed anytime between 6am on Monday to 6am on Wednesday, her exposure window length would be 2 days.
##'
##' @usage data(exp.win.lengths)
##' @source Lessler J et al.  New England Journal of Medicine. Outbreak of 2009 Pandemic Influenza A (H1N1) at a New York City School. 2009. 361(27):2628-2636. \doi{10.1056/NEJMoa0906089}
##'
##' @examples
##' data(exp.win.lengths)
##' summary(exp.win.lengths)
##' hist(exp.win.lengths)
##' @keywords datasets
NULL

##' @name fluA.inc.per
##' @docType data
##' @title Coarse incubation period data for influenza A
##' @description These observations on the incubation period of influenza A come from a variety of sources, and were gathered for a literature review.  They report doubly interval-censored, single interval-censored or exact observations for the incubation period.
##' @usage data(fluA.inc.per)
##' @format A data frame with 151 observations on the following 7 variables.
##' \describe{
##' \item{\code{author}}{the name of the primary author for the source of the observation}
##' \item{\code{year}}{the year of the study which is the source of the observation}
##' \item{\code{EL}}{the earliest possible time of infection}
##' \item{\code{ER}}{the latest possible time of infection}
##' \item{\code{SL}}{the earliest possible time of symptom onset}
##' \item{\code{SR}}{the latest possible time of symptom onset}
##' \item{\code{type}}{an indicator of the type of observation: 0 for doubly interval-censored, 1 for single-interval censored, 2 for exact}}
##' @source Lessler J, Reich NG, Brookmeyer R, Perl TM, Nelson KE, Cummings DAT. (2009) A systematic review of the incubation periods of acute respiratory viral infections. Lancet Infectious Diseases. 9(5):291-300.
##' @examples
##' data(fluA.inc.per)
##' head(fluA.inc.per)
##' @keywords datasets
NULL

##' @name simulated.outbreak.deaths
##' @docType data
##' @title Simulated case and death reports from a fictional outbreak
##' @description This dataset provides reported counts of cases and deaths occurring at different time points across a simulated outbreak. Details of the data simulation algorithm are provided in the manuscript "Estimating case fatality ratios from infectious disease surveillance data" (Reich et al., under review, available upon request).
##' @usage data(simulated.outbreak.deaths)
##' @format
##' \describe{
##'             \item{\code{time}}{time, t,  after start of outbreak}
##'             \item{\code{grp}}{an categorical variable indicating membership in one of two groups of a covariate, j}
##'             \item{\code{R}}{number of recovered cases reported at the given t and j}
##'             \item{\code{D}}{number of deaths reported at the given t and j}
##'             \item{\code{N}}{total number of cases and deaths reported at t and j, or D+R}
##'             }
##' @source Reich NG, Lessler J, Cummings DAT, Brookmeyer R. Estimating case fatality ratios from infectious disease surveillance data. Biometrics. 2012. 68(2): 598-606.
##' @examples
##' data(simulated.outbreak.deaths)
##' head(simulated.outbreak.deaths)
##' plot(simulated.outbreak.deaths[simulated.outbreak.deaths[,"grp"]==1,"D"], type="l")
NULL

#' @name nycH1N1
##' @docType data
##' @title Incubation period data from New York City Public Schools, 2009 H1N1 influenza outbreak
##' @description These observations on the incubation period of influenza A come from the investigation of the H1N1 outbreak in NYC schools in the spring of 2009. They report doubly interval-censored observations for the incubation period.
##' @usage data(nycH1N1)
##' @format A data frame with 134 observations on the following 5 variables.
##' \describe{
##' \item{\code{EL}}{the earliest possible time of infection}
##' \item{\code{ER}}{the latest possible time of infection}
##' \item{\code{SL}}{the earliest possible time of symptom onset}
##' \item{\code{SR}}{the latest possible time of symptom onset}
##' \item{\code{type}}{an indicator of the type of observation: 0 for doubly interval-censored, 1 for single-interval censored, 2 for exact. All of these observations are doubly interval-censored.}}
##' @source Lessler J, Reich NG, Cummings DAT and The DOHMH Swine Influenza Investigation Team. Outbreak of 2009 Pandemic Influenza A (H1N1) at a New York City School. New England Journal of Medicine. 2009. 361(27):2628-2636.
##' @examples
##' data(nycH1N1)
##' head(nycH1N1)
##' \dontrun{dic.fit(nycH1N1)}
##' @keywords datasets
NULL

######################################
## code for sample size simulations ##
## Nicholas Reich
## August 2010
######################################

## this is the central user function
## N = overall sample size
## med = median of log normal distribution
## disp = dispersion of log normal distribution
## percentile = what percentile(s) should be simulated
## nsim = how many simulations to run
## exact data = T/F should simulation be done with exact data
## pct.type.A = percent type A data, will be rounded up to nearest integer
## exp.win.dat = a vector of exposure window lengths to sample from
## verb = whether to print 10 iteration counts during the course of the sim


##' @name precision.simulation
##' @aliases precision.simulation.exact
##' @aliases precision.simulation.coarse
##' @aliases generate.coarse.data
##'
##' @title Simulate incubation period analyses with coarse data
##'
##' @description These functions simulate coarse incubation period data sets and
##'   analyze them.  The goal is for these simulations to provide evidence for
##'   how much information a given dataset contains about a characteristic of
##'   the incubation period distribution.
##'
##' @param N Overall sample size for the datasets to be simulated.
##' @param med Median for the assumed log normal distribution of the incubation
##'   periods.
##' @param disp Dispersion for the assumed log normal distribution of the
##'   incubation periods.
##' @param percentile Percentile of the incubation period distribution which we
##'   want to estimate.
##' @param nsim Number of datasets to analyze in the simulation.
##' @param exact.data Either TRUE/FALSE.  Incidates whether the data generated
##'   should be coarsened at all.  If TRUE, pct.type.A and exp.win.dat are
##'   ignored.
##' @param pct.type.A Percent of the N observations that are assumed to be type
##'   A data.  If N*pct.type.A is not an integer, it will be rounded to the
##'   nearest integer.
##' @param exp.win.dat A vector of exposure window lengths.  Defaults to the
##'   observed window lengths from Lessler et al. (see below).
##' @param verb If TRUE, a message with the system time and iteration number
##'   will be printed ten times during the simulation run.
##'
##'
##' @rdname precision.simulation
##' @return The \code{precision.simulation} functions return a matrix with four
##'   columns and nsim rows.  The "ests" column gives the estimated percentiles
##'   for the incubation period distribution.  The "SE" column gives the
##'   standard error for the estimate.  The "conv" column is 1 if the doubly
##'   interval-censored likelihood maximization converged.  Otherwise, it is 0.
##'   The "bias" column gives the estimated percentile - true percentile. The
##'   \code{generate.coarse.data} function returns a matrix with data suitable
##'   for analysis by the \code{dic.fit} function.
##' @export
precision.simulation <- function(N,
                                 med = 2,
                                 disp = 1.3,
                                 percentile = 0.5,
                                 nsim = 100,
                                 exact.data = FALSE,
                                 pct.type.A = 0.5,
                                 exp.win.dat = NULL,
                                 verb = FALSE) {
  ## logic check
  if (percentile <= 0 || percentile >= 1) {
    stop("percentile must be between 0 and 1.")
  }
  if (pct.type.A < 0 || pct.type.A > 1) {
    stop("% of data that is type A must be between 0 and 1.")
  }
  if (is.null(exp.win.dat)) {
    if (verb) message("NYC exposure window data used")
    exp.win.dat <- get(data(exp.win.lengths, envir = environment()))
  }

  ## TODO: add default exp.win.dat to package and load if NULL

  if (exact.data) {
    out <- precision.simulation.exact(
      N = N,
      med = med,
      disp = disp,
      percentile = percentile,
      nsim = nsim,
      verb = verb
    )
  } else {
    out <- precision.simulation.coarse(
      N = N,
      med = med,
      disp = disp,
      percentile = percentile,
      nsim = nsim,
      pct.type.A = pct.type.A,
      exp.win.dat = exp.win.dat,
      verb = verb
    )
  }

  target <- qlnorm(percentile, log(med), log(disp))
  bias <- out[, "ests"] - target
  out <- cbind(out, bias)

  return(out)
}

##' @rdname precision.simulation
##' @export
precision.simulation.exact <- function(N,
                                       med,
                                       disp,
                                       percentile,
                                       nsim,
                                       verb) {
  storage <- matrix(NA, ncol = 3, nrow = nsim)
  colnames(storage) <- c("ests", "SE", "conv")

  data <- matrix(0, ncol = 6, nrow = nsim * N)
  colnames(data) <- c("dataset.id", "EL", "ER", "SL", "SR", "type")
  data[, "dataset.id"] <- rep(1:nsim, each = N)
  data[, "SL"] <- rlnorm(N * nsim, meanlog = log(med), sdlog = log(disp))
  data[, "SR"] <- data[, "SL"]
  data[, "type"] <- 2

  for (i in 1:nsim) {
    tmp.dat <- data[which(data[, "dataset.id"] == i), ]
    tmp.fit <- dic.fit(tmp.dat, ptiles = percentile)
    if (tmp.fit$conv == 1) {
      row.name <- paste0("p", round(percentile * 100))
      which.row <-
        which(rownames(tmp.fit$ests) == row.name)[1]
      storage[i, c("ests", "SE")] <-
        tmp.fit$ests[which.row, c("est", "StdErr")]
    } else {
      storage[i, c("ests", "SE")] <- NA
    }
    storage[i, "conv"] <- tmp.fit$conv
    if (verb && i %% (round(nsim / 10)) == 0) {
      print(paste("iteration", i, "complete ::", Sys.time()))
    }
  }
  return(storage)
}

##' @rdname precision.simulation
##' @export
precision.simulation.coarse <- function(N,
                                        med,
                                        disp,
                                        percentile,
                                        nsim,
                                        pct.type.A,
                                        exp.win.dat,
                                        verb) {
  ## create storage
  storage <- matrix(NA, ncol = 3, nrow = nsim)
  colnames(storage) <- c("ests", "SE", "conv")

  for (i in 1:nsim) {
    tmp.dat <- generate.coarse.data(
      N = N,
      med = med,
      disp = disp,
      pct.type.A = pct.type.A,
      exp.win.dat = exp.win.dat
    )
    tmp.fit <- dic.fit(tmp.dat, ptiles = percentile)
    if (tmp.fit$conv == 1) {
      row.name <- paste0("p", round(percentile * 100))
      which.row <-
        which(rownames(tmp.fit$ests) == row.name)[1]
      storage[i, c("ests", "SE")] <-
        tmp.fit$ests[which.row, c("est", "StdErr")]
    } else {
      storage[i, c("ests", "SE")] <- NA
    }
    storage[i, "conv"] <- tmp.fit$conv
    if (verb && i %% (round(nsim / 10)) == 0) {
      print(paste("iteration", i, "complete ::", Sys.time()))
    }
  }
  return(storage)
}


##' @rdname precision.simulation
##' @export
generate.coarse.data <- function(N, med, disp, pct.type.A, exp.win.dat) {
  n.type.A <- round(N * pct.type.A)
  n.type.B <- N - n.type.A


  E <- runif(N, 10, 11)
  S <- rlnorm(N, log(med), log(disp)) + E
  SR <- ceiling(S)
  SL <- floor(S)

  ## generate window types
  ##   0 = short with bounded ER = type A
  ##   1 = "long" with no ER = type B
  win.type <- rep(0:1, times = c(n.type.A, n.type.B))

  ## generate window lengths,
  potential.lengths <- sample(exp.win.dat, size = N, replace = TRUE)
  win.length <- 1 * (win.type == 0) + potential.lengths * (win.type > 0)

  ## fix data
  ER <- ceiling(E) * (win.type == 0) + SR * (win.type == 1)
  EL <- pmin(ER - win.length, floor(E))

  ## return the data
  cbind(EL, E, ER, SL, S, SR, win.length, win.type, type = 0)
}

##' censored survival data
##'
##' \code{dic.fit} fits a parametric accelerated failure time model to survival
##' data. It was developed with the application to estimating incubation periods of infectious diseases
##' in mind but is applicable to many general problems.
##' The data can be a mixture of doubly interval-censored, single
##' interval-censored or exact observations from a single univariate
##' distribution. Currently, three distributions are supported: log-normal,
##' gamma, and Weibull. (The Erlang distribution is supported in the
##' \code{dic.fit.mcmc} function, which implements an MCMC version of this
##' code.) We use a consistent (par1, par2) notation for each distribution, they
##' map in the following manner: \deqn{Log-normal(meanlog=par1, sdlog=par2)}
##' \deqn{Gamma(shape=par1, scale=par2)} \deqn{Weibull(shape=par1, scale=par2)}
##' Standard errors of parameters can be computed using closed-form asymptotic
##' formulae or using a bootstrap routine for log-normal and gamma models.
##' Currently, bootstrap SEs are the only option for the gamma models, which do
##' not have a closed form for the percentiles. \code{dic.fit()} calculates
##' asymptotic SEs by default, or whenever the \code{n.boots} option is set to
##' 0. To compute bootstrap SEs, just set \code{n.boots} to be greater than
##' zero. \code{\link{dic.fit.mcmc}} also allows for Markov Chain Monte Carlo
##' fitting of these three parametric models and Erlang models as well.
##'
##'
##' @param dat a matrix with columns named "EL", "ER", "SL", "SR", corresponding
##'   to the left (L) and right (R) endpoints of the windows of possible
##'   exposure (E) and symptom onset (S). Also, a "type" column must be
##'   specified and have entries with 0, 1, or 2, corresponding to doubly
##'   interval-censored, single interval-censored or exact observations,
##'   respectively.
##' @param start.par2 starting value for 2nd parameter of desired distribution
##' @param opt.method method used by optim
##' @param par1.int the log-scale interval of possible median values (in the
##'   same units as the observations in dat).  Narrowing this interval can help
##'   speed up convergence of the algorithm, but care must be taken so that
##'   possible values are not excluded or that the maximization does not return
##'   a value at an endpoint of this interval.
##' @param par2.int the log-scale interval of possible dispersion values
##' @param ptiles percentiles of interest
##' @param dist what distribution to use to fit the data. Default "L" for
##'   log-normal. "G" for gamma, and "W" for Weibull.
##' @param n.boots number of bootstrap resamples (0 means that asymptotic results are desired)
##' @param ... additional options passed to optim
##' @return a cd.fit S4 object.
##' @importFrom methods is
##' @seealso \code{\link{cd.fit}}, \code{\link{dic.fit.mcmc}}
##' @export
##' @examples
##' data(fluA.inc.per)
##' dic.fit(fluA.inc.per, dist="L")
##' @references Reich NG et al.  Statistics in Medicine.  Estimating incubation
##'   periods with coarse data. 2009.
##'   \url{https://pubmed.ncbi.nlm.nih.gov/19598148/}

dic.fit <- function(dat,
                    start.par2 = log(2),
                    opt.method = "L-BFGS-B",
                    par1.int = log(c(0.5, 13)),
                    par2.int = log(c(1.01, log(5))),
                    ptiles = c(0.05, 0.95, 0.99),
                    dist = "L",
                    n.boots = 0,
                    ...) {
  ## check format of dat
  check.data.structure(dat)

  ## check to make sure distribution is supported
  if (!dist %in% c("G", "W", "L")) stop("Please use one of the following distributions Log-Normal (L) , Weibull (W), or Gamma (G)")

  ## no asymptotic results for gamma disribution at the moment so will need bootstrap to be larger tha 0 if dist != "L"
  if (dist == "G" && n.boots <= 0) stop("You must use bootstraping with this distrbution at the moment.  Please increase n.boots to something larger than 0")

  ## check if ptiles are valid
  if (any(ptiles >= 1, ptiles <= 0)) stop("Sorry the percentiles you are requesting are not valid.")

  ## fix sample size
  n <- nrow(dat)

  ## make sure dat is a matrix
  dat <- as.matrix(dat[, c("EL", "ER", "SL", "SR", "type")])
  if (is(dat, "data.frame")) stop("dat should be a matrix.")

  ## find starting values for DIC analysis using profile likelihoods
  start.par1 <- optimize(
    f = pl.par1, interval = par1.int,
    par2 = start.par2, dat = dat, dist = dist
  )$min
  start.par2 <- optimize(
    f = pl.par2, interval = par2.int, par1 = start.par1,
    dat = dat, dist = dist
  )$min

  # cat("start.par1:", start.par1, " start.par2", start.par2, "\n") ##DEBUG

  ## find MLEs for doubly censored data using optim
  tmp <- list(convergence = 1)
  msg <- NULL
  fail <- FALSE

  tmp <- optim(
    par = c(start.par1, start.par2),
    method = opt.method, hessian = TRUE,
    # lower=c(log(0.5), log(log(1.04))),
    fn = loglikhd, dat = dat, dist = dist, ...
  )

  # LEADS TO BETTER ERROR MESSAGES
  #     tryCatch(tmp <- optim(par=c(start.par1, start.par2),
  #                           method=opt.method, hessian=TRUE,
  #                          # lower=c(log(0.5), log(log(1.04))),
  #                           fn=loglikhd, dat=dat,dist=dist, ...),
  #              error = function(e) {
  #                print(e)
  #                  msg <<- e$message
  #                  fail <<- TRUE
  #              },
  #              warning = function(w){
  #                  msg <<- w$message
  #                  fail <<- TRUE
  #              })

  ## also, to catch a few more errors
  if (tmp$convergence != 0 || all(tmp$hessian == 0)) {
    msg <- tmp$message
    if (all(tmp$hessian == 0)) msg <- paste(msg, "& hessian is singular")
    fail <- TRUE
  }


  ## check if optimaization went well
  if (!fail) {
    ## back transform optim fit
    untransformed.fit.params <- dist.optim.untransform(dist, tmp$par)


    ## always going to report median even if not requested
    ptiles.appended <- sort(union(0.5, ptiles))

    ## get asymtotic CIs and SEs
    if (dist == "L" && n.boots <= 0) {
      med <- exp(untransformed.fit.params[1])
      disp <- exp(untransformed.fit.params[2])

      norm.quants <- qnorm(ptiles.appended)
      ests <- c(
        untransformed.fit.params[1],
        untransformed.fit.params[2],
        med * disp^norm.quants
      )
      Sig <- solve(tmp$hessian)
      ses <- dic.getSE(dat = dat, par1 = log(med), log.par2 = log(log(disp)), Sig = Sig, ptiles = ptiles.appended, dist = dist, opt.method = opt.method)
      ## get cis
      cil <- ests - qt(0.975, n - 1) * ses
      cih <- ests + qt(0.975, n - 1) * ses
      ## save the quantile estimates
      quant.matrix <- matrix(c(ests, cil, cih, ses),
        nrow = 2 + length(ptiles.appended), byrow = FALSE
      )
      ptiles.names <- paste0("p", 100 * ptiles.appended)

      rownames(quant.matrix) <- c("meanlog", "sdlog", ptiles.names)
      colnames(quant.matrix) <- c("est", "CIlow", "CIhigh", "StdErr")
    } else if (dist == "W" && n.boots <= 0) {
      shape <- untransformed.fit.params[1]
      scale <- untransformed.fit.params[2]

      ests <- c(
        shape,
        scale,
        scale * (-log(1 - ptiles.appended))^(1 / shape)
      )

      Sig <- solve(tmp$hessian)
      ses <- dic.getSE(
        dat = dat,
        par1 = shape,
        log.par2 = log(scale),
        Sig = Sig,
        ptiles = ptiles.appended,
        dist = dist,
        opt.method = opt.method
      )
      ## get cis
      cil <- ests - qt(0.975, n - 1) * ses
      cih <- ests + qt(0.975, n - 1) * ses

      ## save the quantile estimates
      quant.matrix <- matrix(c(ests, cil, cih, ses),
        nrow = 2 + length(ptiles.appended), byrow = FALSE
      )

      ptiles.names <- paste0("p", 100 * ptiles.appended)

      rownames(quant.matrix) <- c("shape", "scale", ptiles.names)
      colnames(quant.matrix) <- c("est", "CIlow", "CIhigh", "StdErr")
    } else { ## running bootstrap

      Sig <- solve(tmp$hessian)

      ## get estimates and cis for shape and scale
      boot.params <- dic.get.boots(
        dat = dat,
        par1 = untransformed.fit.params[1],
        par2 = untransformed.fit.params[2], # keeping it logged to stay consistent with previous function
        dist = dist,
        opt.method = opt.method,
        n.boots = n.boots
      )

      na.rows <- is.na(rowSums(boot.params))
      ## if we have any bootstraps that we couldn't get the MLE for:
      if (sum(na.rows) > 0) {
        warning(sprintf("Could not estimate the MLEs for %.0f of %.0f bootstrap replications. Excluding these from the calculation of confidence intervals and standard errors so interpret with caution. \n", sum(na.rows), n.boots))
        boot.params <- boot.params[-which(na.rows), ]
      }

      cis.params <- apply(boot.params, 2, function(x) quantile(x, c(0.025, 0.975)))

      ## adding median to  below since the exp(shape) paramter no longer has the nice interpretration
      ## of the log-normal model

      if (dist == "L") {
        boot.funcs <- apply(boot.params, 1, function(x) qlnorm(ptiles.appended, meanlog = x[1], sdlog = x[2]))
        ests <- qlnorm(ptiles.appended, untransformed.fit.params[1], untransformed.fit.params[2])
        param1.name <- "meanlog"
        param2.name <- "sdlog"
      } else if (dist == "W") {
        boot.funcs <- apply(boot.params, 1, function(x) qweibull(ptiles.appended, shape = x[1], scale = x[2]))
        ests <- qweibull(ptiles.appended, shape = untransformed.fit.params[1], scale = untransformed.fit.params[2])
        param1.name <- "shape"
        param2.name <- "scale"
      } else if (dist == "G") {
        boot.funcs <- apply(boot.params, 1, function(x) qgamma(ptiles.appended, shape = x[1], scale = x[2]))
        ests <- qgamma(ptiles.appended, shape = untransformed.fit.params[1], scale = untransformed.fit.params[2])
        param1.name <- "shape"
        param2.name <- "scale"
      }

      ## std deviations of bootstraps for parameters
      sds.params <- apply(boot.params, 2, sd)

      ## get percentile estimates
      cis.ptiles <- apply(boot.funcs, 1, function(x) quantile(x, c(0.025, 0.975)))
      sds.ptiles <- apply(boot.funcs, 1, sd)

      quant.matrix <- matrix(c(untransformed.fit.params, ests, cis.params[1, ], cis.ptiles[1, ], cis.params[2, ], cis.ptiles[2, ], sds.params, sds.ptiles), nrow = 2 + length(ptiles.appended), byrow = FALSE)
      ## deal with row and column names
      ptiles.names <- paste0("p", 100 * ptiles.appended)
      rownames(quant.matrix) <- c(param1.name, param2.name, ptiles.names)
      colnames(quant.matrix) <- c("est", "CIlow", "CIhigh", "SD")
    }

    if ("boot.params" %in% ls()) {
      bp <- data.frame(boot.params)
      ci.method <- "Bootstrap"
    } else {
      bp <- data.frame()
      ci.method <- "Asymptotic"
    }

    return(
      new("cd.fit",
        ests = round(quant.matrix, 3),
        conv = 1,
        MSG = "",
        loglik = -tmp$value,
        samples = bp,
        data = data.frame(dat),
        dist = dist,
        inv.hessian = Sig,
        est.method = "Maximum Likelihood - optim",
        ci.method = ci.method
      )
    )
  } else { ## if optimization fails:

    return(
      new("cd.fit",
        ests = matrix(NA, nrow = 5, ncol = 4),
        conv = 0,
        MSG = msg,
        loglik = numeric(0),
        samples = data.frame(),
        data = data.frame(dat),
        dist = dist,
        inv.hessian = matrix(),
        est.method = "Maximum Likelihood - optim",
        ci.method = ""
      )
    )
  }
}


## profile likelihood for par1 -- used by dic.fit() to get starting values
pl.par1 <- function(par1, par2, dat, dist) {
  loglikhd(pars = c(par1, par2), dist = dist, dat = dat)
}


## profile likelihood for par2 -- used by dic.fit() to get starting values
pl.par2 <- function(par2, par1, dat, dist) {
  loglikhd(pars = c(par1, par2), dist = dist, dat = dat)
}

## functions that manipulate/calculate the likelihood for the censored data
## the functions coded here are taken directly from the
## doubly interval censored likelihood notes.
fw1 <- function(t, EL, ER, SL, SR, par1, par2, dist) {
  ## function that calculates the first function for the DIC integral
  if (dist == "W") {
    (ER - SL + t) * dweibull(x = t, shape = par1, scale = par2)
  } else if (dist == "off1W") {
    (ER - SL + t) * dweibullOff1(x = t, shape = par1, scale = par2)
  } else if (dist == "G") {
    (ER - SL + t) * dgamma(x = t, shape = par1, scale = par2)
  } else if (dist == "off1G") {
    (ER - SL + t) * dgammaOff1(x = t, shape = par1, scale = par2)
  } else if (dist == "L") {
    (ER - SL + t) * dlnorm(x = t, meanlog = par1, sdlog = par2)
  } else if (dist == "off1L") {
    (ER - SL + t) * dlnormOff1(x = t, meanlog = par1, sdlog = par2)
  } else {
    stop("distribution not supported")
  }
}


fw3 <- function(t, EL, ER, SL, SR, par1, par2, dist) {
  ## function that calculates the third function for the DIC integral
  if (dist == "W") {
    (SR - EL - t) * dweibull(x = t, shape = par1, scale = par2)
  } else if (dist == "off1W") {
    (SR - EL - t) * dweibullOff1(x = t, shape = par1, scale = par2)
  } else if (dist == "G") {
    (SR - EL - t) * dgamma(x = t, shape = par1, scale = par2)
  } else if (dist == "off1G") {
    (SR - EL - t) * dgammaOff1(x = t, shape = par1, scale = par2)
  } else if (dist == "L") {
    (SR - EL - t) * dlnorm(x = t, meanlog = par1, sdlog = par2)
  } else if (dist == "off1L") {
    (SR - EL - t) * dlnormOff1(x = t, meanlog = par1, sdlog = par2)
  } else {
    stop("distribution not supported")
  }
}


lik <- function(par1, par2, EL, ER, SL, SR, type, dist) {
  ## returns the right likelihood for the type of data
  ## 0 = DIC, 1=SIC, 2=exact
  if (type == 0) {
    return(diclik2(par1, par2, EL, ER, SL, SR, dist))
  }
  if (type == 1) {
    return(siclik(par1, par2, EL, ER, SL, SR, dist))
  }
  if (type == 2) {
    return(exactlik(par1, par2, EL, ER, SL, SR, dist))
  }
}


## calculates the DIC likelihood by integration
diclik <- function(par1, par2, EL, ER, SL, SR, dist) {
  ## if symptom window is bigger than exposure window
  if (SR - SL > ER - EL) {
    dic1 <- integrate(fw1,
      lower = SL - ER, upper = SL - EL,
      subdivisions = 10,
      par1 = par1, par2 = par2,
      EL = EL, ER = ER, SL = SL, SR = SR,
      dist = dist
    )$value
    if (dist == "W") {
      dic2 <- (ER - EL) *
        (pweibull(SR - ER, shape = par1, scale = par2) - pweibull(SL - EL, shape = par1, scale = par2))
    } else if (dist == "off1W") {
      dic2 <- (ER - EL) *
        (pweibullOff1(SR - ER, shape = par1, scale = par2) - pweibullOff1(SL - EL, shape = par1, scale = par2))
    } else if (dist == "G") {
      dic2 <- (ER - EL) *
        (pgamma(SR - ER, shape = par1, scale = par2) - pgamma(SL - EL, shape = par1, scale = par2))
    } else if (dist == "off1G") {
      dic2 <- (ER - EL) *
        (pgammaOff1(SR - ER, shape = par1, scale = par2) - pgammaOff1(SL - EL, shape = par1, scale = par2))
    } else if (dist == "L") {
      dic2 <- (ER - EL) *
        (plnorm(SR - ER, par1, par2) - plnorm(SL - EL, par1, par2))
    } else if (dist == "off1L") {
      dic2 <- (ER - EL) *
        (plnormOff1(SR - ER, par1, par2) - plnormOff1(SL - EL, par1, par2))
    } else {
      stop("distribution not supported")
    }
    dic3 <- integrate(fw3,
      lower = SR - ER, upper = SR - EL,
      subdivisions = 10,
      par1 = par1, par2 = par2,
      EL = EL, ER = ER, SL = SL, SR = SR,
      dist = dist
    )$value
    return(dic1 + dic2 + dic3)
  } else { ## if exposure window is bigger than symptom window
    dic1 <- integrate(fw1,
      lower = SL - ER, upper = SR - ER, subdivisions = 10,
      par1 = par1, par2 = par2,
      EL = EL, ER = ER, SL = SL, SR = SR,
      dist = dist
    )$value
    if (dist == "W") {
      dic2 <- (SR - SL) *
        (pweibull(SL - EL, shape = par1, scale = par2) - pweibull(SR - ER, shape = par1, scale = par2))
    } else if (dist == "off1W") {
      dic2 <- (SR - SL) *
        (pweibullOff1(SL - EL, shape = par1, scale = par2) - pweibullOff1(SR - ER, shape = par1, scale = par2))
    } else if (dist == "G") {
      dic2 <- (SR - SL) *
        (pgamma(SL - EL, shape = par1, scale = par2) - pgamma(SR - ER, shape = par1, scale = par2))
    } else if (dist == "off1G") {
      dic2 <- (SR - SL) *
        (pgammaOff1(SL - EL, shape = par1, scale = par2) - pgammaOff1(SR - ER, shape = par1, scale = par2))
    } else if (dist == "L") {
      dic2 <- (SR - SL) *
        (plnorm(SL - EL, par1, par2) - plnorm(SR - ER, par1, par2))
    } else if (dist == "off1L") {
      dic2 <- (SR - SL) *
        (plnormOff1(SL - EL, par1, par2) - plnormOff1(SR - ER, par1, par2))
    } else {
      stop("distribution not supported")
    }
    dic3 <- integrate(fw3,
      lower = SL - EL, upper = SR - EL,
      subdivisions = 10,
      par1 = par1, par2 = par2,
      EL = EL, ER = ER, SL = SL, SR = SR,
      dist = dist
    )$value
    return(dic1 + dic2 + dic3)
  }
}

## this dic likelihood is designed for data that has overlapping intervals
diclik2 <- function(par1, par2, EL, ER, SL, SR, dist) {
  if (SL > ER) {
    return(diclik(par1, par2, EL, ER, SL, SR, dist))
  } else {
    lik1 <- integrate(diclik2.helper1,
      lower = EL, upper = SL,
      SL = SL, SR = SR, par1 = par1, par2 = par2, dist = dist
    )$value
    lik2 <- integrate(diclik2.helper2,
      lower = SL, upper = ER,
      SR = SR, par1 = par1, par2 = par2, dist = dist
    )$value
    return(lik1 + lik2)
  }
}

## likelihood functions for diclik2
diclik2.helper1 <- function(x, SL, SR, par1, par2, dist) {
  if (dist == "W") {
    pweibull(SR - x, shape = par1, scale = par2) - pweibull(SL - x, shape = par1, scale = par2)
  } else if (dist == "off1W") {
    pweibullOff1(SR - x, shape = par1, scale = par2) - pweibullOff1(SL - x, shape = par1, scale = par2)
  } else if (dist == "G") {
    pgamma(SR - x, shape = par1, scale = par2) - pgamma(SL - x, shape = par1, scale = par2)
  } else if (dist == "off1G") {
    pgammaOff1(SR - x, shape = par1, scale = par2) - pgammaOff1(SL - x, shape = par1, scale = par2)
  } else if (dist == "L") {
    plnorm(SR - x, par1, par2) - plnorm(SL - x, par1, par2)
  } else if (dist == "off1L") {
    plnormOff1(SR - x, par1, par2) - plnormOff1(SL - x, par1, par2)
  } else {
    stop("distribution not supported")
  }
}

diclik2.helper2 <- function(x, SR, par1, par2, dist) {
  if (dist == "W") {
    pweibull(SR - x, shape = par1, scale = par2)
  } else if (dist == "off1W") {
    pweibullOff1(SR - x, shape = par1, scale = par2)
  } else if (dist == "G") {
    pgamma(SR - x, shape = par1, scale = par2)
  } else if (dist == "off1G") {
    pgammaOff1(SR - x, shape = par1, scale = par2)
  } else if (dist == "L") {
    plnorm(SR - x, par1, par2)
  } else if (dist == "off1L") {
    plnormOff1(SR - x, par1, par2)
  } else {
    stop("distribution not supported")
  }
}


siclik <- function(par1, par2, EL, ER, SL, SR, dist) {
  ## calculates the SIC likelihood as the difference in CDFs
  if (dist == "W") {
    pweibull(SR - EL, shape = par1, scale = par2) - pweibull(SL - ER, shape = par1, scale = par2)
  } else if (dist == "off1W") {
    pweibullOff1(SR - EL, shape = par1, scale = par2) - pweibullOff1(SL - ER, shape = par1, scale = par2)
  } else if (dist == "off1G") {
    pgammaOff1(SR - EL, shape = par1, scale = par2) - pgammaOff1(SL - ER, shape = par1, scale = par2)
  } else if (dist == "G") {
    pgamma(SR - EL, shape = par1, scale = par2) - pgamma(SL - ER, shape = par1, scale = par2)
  } else if (dist == "L") {
    plnorm(SR - EL, par1, par2) - plnorm(SL - ER, par1, par2)
  } else if (dist == "off1L") {
    plnormOff1(SR - EL, par1, par2) - plnormOff1(SL - ER, par1, par2)
  } else {
    stop("distribution not supported")
  }
}

exactlik <- function(par1, par2, EL, ER, SL, SR, dist) {
  ## calculates the likelihood for an exact observation

  ## NB: the two Ss should be equal and the two Es should be equal
  ##     so it doesn't matter which pair we use in the forpar1la below.
  if (dist == "W") {
    dweibull(SR - EL, shape = par1, scale = par2)
  } else if (dist == "off1W") {
    dweibullOff1(SR - EL, shape = par1, scale = par2)
  } else if (dist == "off1G") {
    dgammaOff1(SR - EL, shape = par1, scale = par2)
  } else if (dist == "G") {
    dgamma(SR - EL, shape = par1, scale = par2)
  } else if (dist == "L") {
    dlnorm(SR - EL, par1, par2)
  } else if (dist == "off1L") {
    dlnormOff1(SR - EL, par1, par2)
  } else {
    stop("distribution not supported")
  }
}


##' Negative log likelihood for a dataset of interval-censored data, given a
##' distribution and its parameters.
##' @param pars vector of the transformed (estimation scale) parameters
##' @param dat a dataset, as in \code{dic.fit}
##' @param dist a distribution, as in \code{dic.fit}
##'
##' @details This package uses two versions of each parameter, the estimation
##'   scale, or the scale that is used for numerical optimization, and the
##'   reporting scale, or the natural scale of the parameters. For all
##'   likelihood calculations, this \code{loglikhd} function expects parameters
##'   that are on the estimation scale, i.e. have range \eqn{(-\infty, \infty)}.
##'   Specifically, this translates into all parameters for all distributions
##'   being log-transformed except for the meanlog (i.e. "par1") for the
##'   log-normal distribution.
##'
##' @return negative log-likelihood for a given dataset, parameters, and
##'   distribution.
##' @export
loglikhd <- function(pars, dat, dist) {
  # if the distribution is erlanf transform correctly for gamma
  if (dist == "E") {
    return(loglikhd(c(log(pars[1]), pars[2]), dat, dist = "G"))
  }

  ## calculates the log-likelihood of DIC data
  ## dat must have EL, ER, SL, SR and type columns

  ## expecting transformed params from optimiztion
  ## e.g. for log-normal expecting c(meanlog,log(sdlog))
  pars <- dist.optim.untransform(dist, pars)
  par1 <- pars[1]
  par2 <- pars[2]

  ## cat(sprintf("par1 = %.2f, par2 = %.2f \n",par1, par2))  ## for debugging
  n <- nrow(dat)
  totlik <- 0

  for (i in 1:n) {
    # cat(i,"start\n") ##DEBUG
    totlik <- totlik + log(lik(par1, par2,
      type = dat[i, "type"],
      EL = dat[i, "EL"], ER = dat[i, "ER"],
      SL = dat[i, "SL"], SR = dat[i, "SR"],
      dist = dist
    ))
    # cat(i,"end = ", totlik, "\n") ##DEBUG
  }


  return(-totlik) ## NB: NEEDS TO BE -totlik IF WE ARE MAXIMIZING USING OPTIM!
  ## May want to change this name later to reflect that is it negative log lik
}


## calculates the standard errors for estimates from dic.fit() using delta method (NOTE: only works for log-normal and Weibull Models at the moment)
## this function calculates the asymptotic standard errors based on the delta method
## the var/cov matrix has been calculated on the log(par1) and log(par2) scale
## the df objects below represent the gradient matrix of the transformations,
##    for on each distribution, from the parameters on the estimation scale
dic.getSE <- function(par1, log.par2, Sig, ptiles, dist, dat, opt.method) {
  cat(sprintf("Computing Asymptotic Confidence Intervals \n"))
  par2 <- exp(log.par2) # log.par2 input historically so I kept it as is

  if (dist == "L") {
    qnorms <- qnorm(ptiles)
    df <- matrix(
      c(
        1, 0, exp(par1 + qnorms * par2),
        0, par2, qnorms * exp(par1 + qnorms * par2 + log.par2)
      ),
      nrow = 2, ncol = 2 + length(ptiles), byrow = TRUE
    )
  } else if (dist == "W") {
    df <- matrix(
      c(
        par1, 0, par1 * (-log(1 - ptiles))^(1 / par2), # d/d log(par1)
        0, par2, -par1 / par2 * (-log(1 - ptiles))^(1 / par2) * log(-log(1 - ptiles))
      ), # d/d log(par2)
      nrow = 2, ncol = 2 + length(ptiles), byrow = TRUE
    )
  }
  ses <- sqrt(diag(t(df) %*% Sig %*% df))
  return(ses)
}

## returns matrix of bootstrap estimates of untransformed parameters for distrbution
dic.get.boots <- function(par1, par2, dist, dat, opt.method, n.boots = 100) {
  cat(sprintf("Bootstrapping (n=%i) Standard Errors for %s \n", n.boots, dist))
  boots <- vector("list", n.boots)

  ## sample line numbers from the data
  line.nums <- matrix(sample(seq_len(nrow(dat)), nrow(dat) * n.boots, replace = TRUE), nrow = nrow(dat), ncol = n.boots)
  ## set up progress bar
  pb <- txtProgressBar(min = 0, max = n.boots, style = 3)
  for (i in 1:n.boots) {
    boots[[i]] <-
      single.boot(par1.s = par1, par2.s = par2, opt.method = opt.method, dat.tmp = dat[line.nums[, i], ], dist = dist)
    setTxtProgressBar(pb, i)
  }
  close(pb)

  ## grab the params from each
  ## remember if any failed there will be NAs here
  par1s <- sapply(boots, function(x) x$par[1])
  par2s <- sapply(boots, function(x) x$par[2])

  return(cbind(par1 = par1s, par2 = par2s))
}

## estimates one set of parameters for a single bootstrap resample
## returns optim list object with estimates for the untransformed two parameters of the specified dist
single.boot <- function(par1.s, par2.s, opt.method, dat.tmp, dist, ...) {
  tmp <- list(convergence = 1)
  pars.transformed <- dist.optim.transform(dist, c(par1.s, par2.s))
  tryCatch(
    tmp <- optim(
      par = pars.transformed,
      method = opt.method,
      hessian = FALSE,
      fn = loglikhd,
      dat = dat.tmp, dist = dist, ...
    ),
    error = function(e) {
      NULL
    },
    warning = function(w) {
      NULL
    }
  )

  if (tmp$convergence != 0 || all(tmp$hessian == 0)) {
    msg <- tmp$message
    if (all(tmp$hessian == 0)) msg <- paste(msg, "& hessian is singular")
  }

  ## transform back to original scale
  ## return NAs if we can't find the min for this param set
  if (is.null(tmp$par)) {
    tmp$par <- c(NA, NA)
  } else {
    tmp$par <- dist.optim.untransform(dist, tmp$par)
  }

  return(tmp)
}


## Transforms parameters of a specific distriution for unbounded optimization
## returns vector of transformed parameters
dist.optim.transform <- function(dist, pars) {
  if (dist %in% c("G", "off1G")) {
    return(log(pars)) # for shape and scale
  } else if (dist %in% c("W", "off1W")) {
    return(log(pars)) # for shape and scale
  } else if (dist == "E") {
    # shape not transformed, logged
    return(c(pars[1], log(pars[2])))
  } else if (dist %in% c("L", "off1L")) {
    return(c(pars[1], log(pars[2]))) # for meanlog, sdlog
  } else {
    stop(sprintf("Distribtion (%s) not supported", dist))
  }
}

## Untransforms parameters before entering likelihood
## returns vector of untransformed parameters
dist.optim.untransform <- function(dist, pars) {
  if (dist %in% c("G", "off1G")) {
    return(exp(pars)) # for shape and scale
  } else if (dist %in% c("W", "off1W")) {
    return(exp(pars)) # for shape and scale
  } else if (dist == "E") {
    # shape identity, scale logged in estimation scale
    return(c(pars[1], exp(pars[2])))
  } else if (dist %in% c("L", "off1L")) {
    return(c(pars[1], exp(pars[2]))) # for meanlog, sdlog
  } else {
    stop(sprintf("Distribtion (%s) not supported", dist))
  }
}

## Issues a stop if the data does not conform with the expected structure
check.data.structure <- function(dat) {
  ## check format of dat
  cnames <- colnames(dat)
  if (!("EL" %in% cnames)) stop("dat must have column named EL")
  if (!("ER" %in% cnames)) stop("dat must have column named ER")
  if (!("SL" %in% cnames)) stop("dat must have column named SL")
  if (!("SR" %in% cnames)) stop("dat must have column named SR")

  if (!("type" %in% cnames)) stop("dat must have column named type")
  if (!all(dat[, "type"] %in% c(0, 1, 2))) {
    stop("values in type column must be either 0, 1 or 2.")
  }

  if (any(is.na(dat[, c("EL", "ER", "SL", "SR", "type")]))) stop("Missing (NA) values not permitted")
  return(NULL)
}


##' Function that calculates pgamma with a offset of 1 (i.e., 1 is equivalent to 0)
##'
##' @param x value to calculate pgamma at
##' @param replace0 should we replace 0 with epsilon
##' @param ... other parameters to pgamma
##'
##' @return pgamma offset
##'
pgammaOff1 <- function(x, replace0 = FALSE, ...) {
  rc <- pgamma(x - 1, ...)
  if (replace0 && any(rc <= 0)) {
    rc[which(rc <= 0)] <- 10^-8
  }
  return(rc)
}

plnormOff1 <- function(x, replace0 = FALSE, ...) {
  rc <- plnorm(x - 1, ...)
  if (replace0 && any(rc <= 0)) {
    rc[which(rc <= 0)] <- 10^-8
  }
  return(rc)
}

pweibullOff1 <- function(x, replace0 = FALSE, ...) {
  rc <- pweibull(x - 1, ...)
  if (replace0 && any(rc <= 0)) {
    rc[which(rc <= 0)] <- 10^-8
  }
  return(rc)
}




##' Function that calculates dgamma with a offset of 1 (i.e., 1 is equivalent to 0)
##'
##' @param x value to calculate dgamma at
##' @param replace0 should we replace 0 with epsilon
##' @param ... other parameters to dgamma
##'
##' @return dgamma offset
##'
dgammaOff1 <- function(x, replace0 = FALSE, ...) {
  rc <- dgamma(x - 1, ...)

  if (replace0 && rc <= 0) {
    rc <- 10^-8
  }
  return(rc)
}

dlnormOff1 <- function(x, replace0 = FALSE, ...) {
  rc <- dlnorm(x - 1, ...)

  if (replace0 && rc <= 0) {
    rc <- 10^-8
  }
  return(rc)
}

dweibullOff1 <- function(x, replace0 = FALSE, ...) {
  rc <- dweibull(x - 1, ...)

  if (replace0 && rc <= 0) {
    rc <- 10^-8
  }
  return(rc)
}

##' An S4 Class that stores a fitted coarse data object
##'
##'
##' @description This is the output from \code{dic.fit()}, which contains the important bits of information about the model fit and key options used.
##'
##'
##' @section Slots:
##'  \describe{
##'    \item{\code{ests}:}{Matrix of class \code{"numeric"}. This matrix summarizes the results of fitting the model. Rows correspond to the first parameter , the second parameter and then percentiles specified by the ptiles argument. Columns correspond to the point estimate, the lower and upper bounds on the 95\% confidence interval and the standard error of the point estimate. If the maximization does not converge, this matrix is filled with NAs.}
##'    \item{\code{conv}:}{Object of class \code{"numeric"}. A value of 1 indicates successful convergence; 0 indicates unsuccessful convergence.}
##'    \item{\code{MSG}:}{Object of class \code{"character"}. The error message returned from \code{optim()} if the routine fails to converge.}
##'    \item{\code{loglik}:}{Object of class \code{"numeric"}. Value of the estimated maximum log-likelihood.}
##'    \item{\code{samples}:}{Object of class \code{"data.frame"}. Data frame of bootstrap estimates of parameters (if bootstraps were performed).}
##'    \item{\code{data}:}{Object of class \code{"data.frame"}. Original data used to fit model.}
##'    \item{\code{dist}:}{Object of class \code{"character"}. Failure time distribution fit to data. "L" for log-normal, "G" for gamma, "W" for Weibull, and "E" for Erlang.}
##'    \item{\code{inv.hessian}:}{Object of class \code{"matrix"}. The inverse of the hessian matrix for the likelihood surface at the MLE. Used to determine the standard errors for the percentiles. Note that optimization is done on a transformed scale with all parameters logged for all distributions except the first parameter of the log-normal distribution.}
##'    \item{\code{est.method}:}{Object of class \code{"character"}. Method used for estimation.}
##'    \item{\code{ci.method}:}{Object of class \code{"character"}. Method used for estimation of confidence/credible intervals.}
##'  }
##'
##' @name cd.fit
##' @rdname cd.fit
##' @aliases cd.fit-class
##' @exportClass cd.fit
setClass(
  "cd.fit",
  representation(
    ests = "matrix",
    conv = "numeric",
    MSG = "character",
    loglik = "numeric",
    samples = "data.frame",
    data = "data.frame",
    dist = "character",
    inv.hessian = "matrix",
    est.method = "character",
    ci.method = "character"
  )
)


##' An S4 Class that stores a MCMC fit coarse data object
##'
##'
##' This is the output from \code{dic.fit.mcmc()}, which contains the important bits of information about the model fit and key options used.
##'
##'
##' @section Slots:
##'  \describe{
##'    \item{\code{ests}:}{Matrix of class \code{"numeric"}. This matrix summarizes the results of fitting the model. Rows correspond to the first parameter , the second parameter and then percentiles specified by the ptiles argument. Columns correspond to the point estimate, the lower and upper bounds on the 95\% credible interval and the standard error of the point estimate.}
##'    \item{\code{conv}:}{Object of class \code{"numeric"}. Not used in with \code{dic.fit.mcmc}}
##'    \item{\code{MSG}:}{Object of class \code{"character"}. The error message returned from optim() if the routine fails to converge.}
##'    \item{\code{loglik}:}{Object of class \code{"numeric"}.  Not used in with \code{dic.fit.mcmc}.}
##'    \item{\code{samples}:}{Object of class \code{"data.frame"}. Data frame of posterior draws of parameters.}
##'    \item{\code{data}:}{Object of class \code{"data.frame"}. Original data used to fit model.}
##'    \item{\code{dist}:}{Object of class \code{"character"}. Failure time distribution fit to data. "L" for log-normal, "G" for gamma, "W" for Weibull, and "E" for Erlang.}
##'    \item{\code{inv.hessian}:}{Object of class \code{"matrix"}. Not used in with \code{dic.fit.mcmc}.}
##'    \item{\code{est.method}:}{Object of class \code{"character"}. Method used for estimation.}
##'    \item{\code{ci.method}:}{Object of class \code{"character"}. Method used for estimation of confidence/credible intervals.}
##'  }
##'
##' @name cd.fit.mcmc
##' @rdname cd.fit.mcmc
##' @aliases cd.fit.mcmc-class
##' @exportClass cd.fit.mcmc
setClass("cd.fit.mcmc",
  contains = "cd.fit"
)

## we don't want boots and data printing out all the time
setMethod("show", "cd.fit", function(object) {
  cat(sprintf("Coarse Data Model Parameter and Quantile Estimates: \n"))
  print(object@ests)
  cat(sprintf("\n-2*Log Likelihood = %.1f \n", -2 * object@loglik))
  if (object@conv == 0) warning("This model did not converge. Try different starting values or increase the number of iterations")
  if (object@dist == "L" & nrow(object@inv.hessian) > 0) {
    se.dispersion <- sqrt(object@inv.hessian[2, 2] * (exp(object@ests[2, 1] + log(object@ests[2, 1])))^2) # using delta method
    cat(sprintf(
      "\nNote: dispersion parameter is exp(sdlog). In this case it is %.3f (95%% CI %.3f-%.3f). \n",
      exp(object@ests[2, 1]),
      exp(object@ests[2, 1]) - qt(0.975, nrow(object@data) - 1) * se.dispersion,
      exp(object@ests[2, 1]) + qt(0.975, nrow(object@data) - 1) * se.dispersion
    ))
  }
})

## we don't want boots and data printing out all the time
setMethod("show", "cd.fit.mcmc", function(object) {
  cat(sprintf("Coarse Data Model Parameter and Quantile Estimates: \n"))
  print(object@ests)
  if (object@dist == "L") cat(sprintf("\n Note: dispersion parameter is exp(sdlog). In this case it is %.3f. \n", exp(object@ests[2, 1])))
  cat(sprintf("Note: please check that the MCMC converged on the target distribution by running multiple chains. MCMC samples are available in the mcmc slot (e.g. my.fit@mcmc) \n"))
})

#' Get the log-likelihood value of a \code{cd.fit} or \code{cd.fit.mcmc} object
#'
#' @param object A \code{cd.fit} or \code{cd.fit.mcmc} object.
#'
#' @return log-likelihood value
#'
#' @rdname logLik-methods
#' @aliases logLik logLik,cd.fit-method
#' @export
#' @importFrom stats logLik
setMethod(
  "logLik",
  "cd.fit",
  function(object) {
    object@loglik
  }
)

##' Tries to guess the observation types (SIC, DIC, or exact).
##'
##' @param dat a matrix of data, similar to what needs to be passed to \code{dic.fit()}.
##' @return vector of guessed types
##' @export
get.obs.type <- function(dat) {
  type <- rep(0, nrow(dat))
  ## get the single interval censored
  type[dat[, "EL"] == dat[, "ER"]] <- 1
  type[dat[, "SL"] == dat[, "SR"]] <- 1
  type[dat[, "ER"] >= dat[, "SL"]] <- 1

  ## some of those are actually exact!
  type[(dat[, "EL"] == dat[, "ER"]) & (dat[, "SL"] == dat[, "SR"])] <- 2
  return(type)
}

##' Fits the distribution to the passed-in data using MCMC
##' as implemented in MCMCpack.
##'
##' Similar to \code{\link{dic.fit}} but uses MCMC instead of a direct likelihood optimization routine to fit the model. Currently, four distributions are supported: log-normal, gamma, Weibull, and Erlang. See Details for prior specification.
##'
##' The following models are used:
##' \deqn{Log-normal model: f(x) = \frac{1}{x*\sigma \sqrt{2 * \pi}} exp\{-\frac{(\log x - \mu)^2}{2 * \sigma^2}\}}
##' \deqn{Log-normal Default Prior: \mu ~ N(0, 1000), log(\sigma) ~ N(0,1000)}
##' \deqn{Weibull model: f(x) = \frac{\alpha}{\beta}(\frac{x}{\beta})^{\alpha-1} exp\{-(\frac{x}{\beta})^{\alpha}\}}
##' \deqn{Weibull Default Prior Specification: log(\alpha) ~ N( 0, 1000), \beta ~ Gamma(0.001,0.001)}
##' \deqn{Gamma model: f(x) = \frac{1}{\theta^k \Gamma(k)} x^{k-1} exp\{-\frac{x}{\theta}\}}
##'
##' \deqn{Gamma Default Prior Specification: p(k,\theta) \propto \frac{1}{\theta} * \sqrt{k*TriGamma(k)-1}}
##' (Note: this is Jeffery's Prior when both parameters are unknown), and
##' \deqn{Trigamma(x) = \frac{\partial}{\partial x^2} ln(\Gamma(x))}.)
##' \deqn{Erlang model: f(x) = \frac{1}{\theta^k (k-1)!} x^{k-1} exp\{-\frac{x}{\theta}\}}
##' \deqn{Erlang Default Prior Specification: k \sim NBinom(100,1), log(\theta) \sim N(0,1000)}
##' (Note: parameters in the negative binomial distribution above represent mean and size, respectively)
##'
##' @param dat the data
##' @param prior.par1 vector of first prior parameters for each model parameter. If \code{NULL} then default parameters are used (as described in Details section).
##' @param prior.par2 vector of second prior parameters for each model parameter. If \code{NULL} then default parameters are used (as described in Details section).
##' @param init.pars the initial parameter values (vector length = 2 )
##' @param ptiles returned percentiles of the survival survival distribution
##' @param verbose how often do you want a print out from MCMCpack on iteration number and M-H acceptance rate
##' @param burnin number of burnin samples
##' @param n.samples number of samples to draw from the posterior (after the burnin)
##' @param dist distribution to be used (L for log-normal,W for weibull, G for Gamma, and E for erlang, off1G for 1 day right shifted gamma)
##' @param seed seed for the random number generator for MCMC
##' @param ... additional parameters to \link[MCMCpack]{MCMCmetrop1R}
##' @return a cd.fit.mcmc S4 object
##' @importFrom MCMCpack MCMCmetrop1R
##' @export
dic.fit.mcmc <- function(dat,
                         prior.par1 = NULL,
                         prior.par2 = NULL,
                         init.pars = c(1, 1),
                         ptiles = c(0.05, 0.95, 0.99),
                         verbose = 1000, # how often to print update
                         burnin = 3000,
                         n.samples = 5000,
                         dist = "L",
                         seed = NULL,
                         ...) {
  ## check to make sure data is well formed for CDT use:
  check.data.structure(dat)

  ## check to make sure distribution is supported
  if (!dist %in% c("G", "W", "L", "E", "off1G", "off1W", "off1L")) stop("Please use one of the following distributions Log-Normal (L) , Weibull (W), Gamma (G), Erlang (E), offset Log-Normal (off1L), offset Weibull (off1W) or offset Gamma (off1G)")

  ##        ## don't need MCMCpack for Erlang
  ##        if (dist != "E") require(MCMCpack)

  ## check that percentiles are valid
  if (any(ptiles >= 1, ptiles <= 0)) stop("Sorry the percentiles you are requesting are not valid.")

  ## default prior parameters if none specified
  if (is.null(prior.par1)) {
    if (dist %in% c("L", "off1L")) {
      prior.par1 <- c(0, 0)
    } else if (dist %in% c("W", "off1W", "G", "off1G")) {
      prior.par1 <- c(0, 0.001)
    } else if (dist == "E") {
      prior.par1 <- c(100, 0)
    }
  }

  ## default prior parameters if none specified
  if (is.null(prior.par2)) {
    if (dist %in% c("L", "off1L")) {
      prior.par2 <- c(1000, 1000)
    } else if (dist %in% c("W", "off1W", "G", "off1G")) {
      prior.par2 <- c(1000, 0.001)
    } else if (dist == "E") {
      prior.par2 <- c(1, 1000)
    }
  }

  ## random seed based off current time if none specified
  if (is.null(seed)) {
    t <- as.numeric(Sys.time())
    seed <- 1e8 * (t - floor(t))
  }

  cat(sprintf("Running %.0f MCMC iterations \n", n.samples + burnin))

  msg <- NULL
  fail <- FALSE

  ## run the MCMC chains

  ## initial parameters set on reporting scale not estimation scale
  init.pars.trans <- dist.optim.transform(dist, init.pars)


  if (dist != "E") {
    # use MCMC pack for effieciency for distibutions with 2 continuous parameters
    tryCatch(
      mcmc.run <- MCMCmetrop1R(
        fun = mcmcpack.ll,
        theta.init = init.pars.trans,
        burnin = burnin,
        mcmc = n.samples,
        dat = dat,
        prior.par1 = prior.par1,
        prior.par2 = prior.par2,
        verbose = verbose,
        dist = dist,
        logfun = TRUE,
        seed = seed,
        ...
      ),
      error = function(e) {
        msg <<- e$message
        fail <<- TRUE
      },
      warning = function(w) {
        msg <<- w$message
        fail <<- TRUE
      }
    )
  } else {
    # Use the custom metropolis hastings algorithm for erlang
    mcmc.run <- mcmc.erlang(
      dat,
      prior.par1,
      prior.par2,
      init.pars,
      verbose,
      burnin,
      n.samples,
      ...
    )
  }



  if (!fail) {
    ## untransform MCMC parameter draws to natural scale
    untrans.mcmcs <- t(apply(mcmc.run[, 1:2], 1, function(x) dist.optim.untransform(dist = dist, pars = x)))

    ## append median to the percentiles in case it isn't there
    ptiles.appended <- sort(union(0.5, ptiles))
    est.pars <- matrix(nrow = length(ptiles.appended) + 2, ncol = 3)

    if (dist == "L") {
      par1.name <- "meanlog"
      par2.name <- "sdlog"
      mcmc.quantiles <- apply(untrans.mcmcs, 1, function(x) qlnorm(ptiles.appended, meanlog = x[1], sdlog = x[2]))
    } else if (dist == "off1L") {
      par1.name <- "meanlog"
      par2.name <- "sdlog"
      mcmc.quantiles <- apply(untrans.mcmcs, 1, function(x) {
        qlnorm(ptiles.appended, meanlog = x[1], sdlog = x[2]) + 1
      })
    } else if (dist == "G") {
      par1.name <- "shape"
      par2.name <- "scale"
      mcmc.quantiles <- apply(untrans.mcmcs, 1, function(x) qgamma(ptiles.appended, shape = x[1], scale = x[2]))
    } else if (dist == "off1G") {
      par1.name <- "shape"
      par2.name <- "scale"
      mcmc.quantiles <- apply(untrans.mcmcs, 1, function(x) {
        qgamma(ptiles.appended, shape = x[1], scale = x[2]) + 1
      })
    } else if (dist == "W") {
      par1.name <- "shape"
      par2.name <- "scale"
      mcmc.quantiles <- apply(untrans.mcmcs, 1, function(x) qweibull(ptiles.appended, shape = x[1], scale = x[2]))
    } else if (dist == "off1W") {
      par1.name <- "shape"
      par2.name <- "scale"
      mcmc.quantiles <- apply(untrans.mcmcs, 1, function(x) {
        qweibull(ptiles.appended, shape = x[1], scale = x[2]) + 1
      })
    } else if (dist == "E") {
      par1.name <- "shape"
      par2.name <- "scale"
      mcmc.quantiles <- apply(untrans.mcmcs, 1, function(x) qgamma(ptiles.appended, shape = x[1], scale = x[2]))
    } else {
      stop("Sorry, unknown distribution type. Check the 'dist' option.")
      ## not actually needed but just in case
    }

    ## make the return matrix
    colnames(est.pars) <- c("est", "CIlow", "CIhigh")
    rownames(est.pars) <- c(par1.name, par2.name, paste0("p", 100 * ptiles.appended))

    # making the matrix with the actual estimates.
    est.pars[1, ] <- quantile(untrans.mcmcs[, 1], c(0.5, 0.025, 0.975))
    est.pars[2, ] <- quantile(untrans.mcmcs[, 2], c(0.5, 0.025, 0.975))
    cis.ptiles <- t(apply(mcmc.quantiles, 1, function(x) quantile(x, c(0.5, 0.025, 0.975))))
    est.pars[3:nrow(est.pars), 1:3] <- cis.ptiles

    ## finally get tbhe log-likelihood evaluated at the mean posterior for each parameter
    ll <- -loglikhd(pars = dist.optim.transform(dist = dist, est.pars[1:2, 1]), dat = data.frame(dat), dist = dist)

    rc <- new("cd.fit.mcmc",
      ests = round(est.pars, 3),
      conv = numeric(),
      MSG = "",
      loglik = ll,
      samples = data.frame(untrans.mcmcs),
      data = data.frame(dat),
      dist = dist,
      inv.hessian = matrix(),
      est.method = "MCMC",
      ci.method = "MCMC"
    )

    return(rc)
  } else {
    if (msg != "") msg <- paste0("Error: ", msg)
    stop(sprintf("\n%s\nTry adjusting the starting parameters (init.pars), or changing the optimization method (opt.method).", msg))
  }
}



##' posterior log likelihood function to pass to MCMCpack sampler
##'
##' @param pars the parameters to calculate the ll at
##' @param dat the date to base it on
##' @param prior.par1 first parameter of each prior
##' @param prior.par2 second parameter of each prior
##' @param dist the distribution the likelihood is being calculated for
##'
##' @return the posterior log likelihood
mcmcpack.ll <- function(pars,
                        dat,
                        prior.par1,
                        prior.par2,
                        dist) {
  ## get parameters on untransformed scale
  pars.untrans <- dist.optim.untransform(dist, pars)



  if (dist == "L" || dist == "off1L") {
    ## default gamma on scale param and (inproper) uniform on location
    ll <- tryCatch(
      -loglikhd(pars, dat, dist) +
        ## dgamma(pars.untrans[2],shape=par.prior.param1[2],
        ## rate=par.prior.param2[2],log=T),
        sum(dnorm(pars,
          prior.par1,
          prior.par2,
          log = TRUE
        )),
      error = function(e) {
        warning("Loglik failure, returning -Inf")
        return(-Inf)
      }
    )
  } else if (dist == "W" || dist == "off1W") {
    ## using normal prior on the log-shape param and gamma on scale
    ll <- tryCatch(
      -loglikhd(pars, dat, dist) +
        dnorm(pars[1],
          prior.par1[1],
          prior.par2[1],
          log = TRUE
        ) +
        dgamma(pars.untrans[2],
          shape = prior.par1[2],
          rate = prior.par2[2], log = TRUE
        ),
      error = function(e) {
        warning("Loglik failure, returning -Inf")
        return(-Inf)
      }
    )
  } else if (dist == "G" || dist == "off1G") {
    ## using Jeffery's prior
    ll <- tryCatch(
      -loglikhd(pars, dat, dist)
      +
        log(1 / pars.untrans[2] * sqrt(pars.untrans[1] * trigamma(pars.untrans[1]) - 1)),
      error = function(e) {
        warning("Loglik failure, returning -Inf")
        return(-Inf)
      }
    )
  } else {
    stop("Sorry, unknown distribution type. Check the 'dist' option.")
  }
  return(ll)
}

##' Does a metropolis hastings for the Erlang distribution
##'
##' @param dat the data to fit
##' @param prior.par1 mean of priors. A negative binomial (for shape) and a normal for log(scale)
##' @param prior.par2 dispersion parameters for priors, dispersion for negative binomial, log scale sd for normal
##' @param init.pars the starting parameters on the reporting scale
##' @param verbose how often to print an update
##' @param burnin how many burnin iterations to do
##' @param n.samples the number of samples to keep and report back
##' @param sds the standard deviations for the proposal distribution
##'
##' @return a matrix of n.samples X 2 parameters, on the estimation scale
##'
mcmc.erlang <- function(dat, prior.par1, prior.par2,
                        init.pars, verbose, burnin, n.samples, sds = c(1, 1)) {
  # make a logging return matrix
  states <- matrix(nrow = burnin + n.samples, ncol = 4)
  colnames(states) <- c("shape", "scale", "ll", "accept")

  # calculate the LL for the initial parameters.
  shape.cur <- init.pars[1]
  scale.cur <- log(init.pars[2])
  ll.cur <- -loglikhd(c(log(shape.cur), scale.cur), dat, dist = "G") +
    dnbinom(shape.cur, mu = prior.par1[1], size = prior.par2[1], log = TRUE) +
    dnorm(scale.cur, prior.par1[2], prior.par2[2], log = TRUE)

  if (ll.cur == -Inf) {
    stop("zero starting likelihood. check priors are appropriate for Erlang distribution")
  }

  states[1, ] <- c(shape.cur, scale.cur, ll.cur, 1)

  for (i in 2:(burnin + n.samples)) {
    # propose new parameters
    shape.prop <- shape.cur + round(rnorm(1, 0, sds[1])) # discretized normal
    scale.prop <- scale.cur + rnorm(1, 0, sds[2])

    # calculate the liklihood using the new parameters...go ahead and do -Inf if shape is -
    if (shape.prop < 0) {
      ll.prop <- -Inf
    } else {
      ll.prop <- -loglikhd(c(log(shape.prop), scale.prop), dat, dist = "G") +
        dnbinom(shape.prop, mu = prior.par1[1], size = prior.par2[1], log = TRUE) +
        dnorm(scale.prop, prior.par1[2], prior.par2[2], log = TRUE)
    }

    # accept or reject
    l.alpha <- log(runif(1))


    if ((ll.prop - ll.cur) > l.alpha) {
      # accept
      shape.cur <- shape.prop
      scale.cur <- scale.prop
      ll.cur <- ll.prop
      states[i, ] <- c(shape.cur, scale.cur, ll.cur, 1)
    } else {
      # reject
      states[i, ] <- c(shape.cur, scale.cur, ll.cur, 0)
    }

    if (verbose != 0 && i %% verbose == 0) {
      cat("Erlang MCMC iteration ", i, " of ", burnin + n.samples, "\n")
      cat("LL = ", ll.cur, "\n")
      cat("theta = ", c(shape.cur, scale.cur), "\n")
      cat("acceptance rate = ", sum(states[1:i, 4]) / i, "\n")
    }
  }

  rc <- states[(burnin + 1):(burnin + n.samples), 1:2]
  return(rc)
}

### function defining the EM algorithm for the CFR estimates

##' A function to estimate the relative case fatality ratio when reporting rates
##' are time-varying and deaths are lagged because of survival time.
##'
##' This function implements an EM algorithm to estimate the relative case
##' fatality ratio between two groups when reporting rates are time-varying and
##' deaths are lagged because of survival time.
##' @usage EMforCFR(assumed.nu, alpha.start.values, full.data, max.iter = 50,
##'   verb = FALSE, tol = 1e-10, SEM.var = TRUE)
##'
##' @param assumed.nu  a vector of probabilities corresponding to the survival
##'   distribution, i.e. nu[i]=Pr(surviving i days | fatal case)
##' @param alpha.start.values a vector starting values for the reporting rate
##'   parameter of the GLM model. This must have length which corresponds to one
##'   less than the number of unique integer values of full.dat[,"new.times"].
##' @param full.data  A matrix of observed data. See description below.
##' @param max.iter The maximum number of iterations for the EM algorithm and
##'   the accompanying SEM algorithm (if used).
##' @param verb An indicator for whether the function should print results as it
##'   runs.
##' @param tol A tolerance to use to test for convergence of the EM algorithm.
##' @param SEM.var If TRUE, the SEM algorithm will be run in addition to the EM
##'   algorithm to calculate the variance of the parameter estimates.
##'
##' @details The data matrix full.data must have the following columns:
##'   \describe{ \item{grp}{a 1 or a 2 indicating which of the two groups, j,
##'   the observation is for.} \item{new.times}{an integer value representing
##'   the time, t, of observation.} \item{R}{the count of recovered cases with
##'   onset at time t in group j.} \item{D}{the count of deaths which occurred at
##'   time t in groupo j (note that these deaths did not have disease onset at
##'   time t but rather died at time t).} \item{N}{the total cases at t, j, or
##'   the sum of R and D columns.} }
##'
##' @return A list with the following elements \describe{ \item{naive.rel.cfr
##'   }{the naive estimate of the relative case fatality ratio}
##'   \item{glm.rel.cfr }{the reporting-rate-adjusted estimate of the relative
##'   case fatality ratio} \item{EM.rel.cfr }{the lag-adjusted estimate of the
##'   relative case fatality ratio} \item{EM.re.cfr.var }{the variance for the
##'   log-scale lag-adjusted estimator taken from the final M-step}
##'   \item{EM.rel.cfr.var.SEM }{ the Supplemented EM algorithm variance for the
##'   log-scale lag-adjusted estimator} \item{EM.rel.cfr.chain }{a vector of the
##'   EM algorithm iterates of the lag-adjusted relative CFR estimates}
##'   \item{EMiter}{the number of iterations needed for the EM algorithm to
##'   converge} \item{EMconv}{indicator for convergence of the EM algorithm.  0
##'   indicates all parameters converged within max.iter iterations.  1 indicates
##'   that the estimate of the relative case fatality ratio converged but other
##'   did not.  2 indicates that the relative case fatality ratio did not
##'   converge.} \item{SEMconv}{indicator for convergence of SEM algorithm.
##'   Same scheme as EMconv.} \item{ests}{ the coefficient estimates for the
##'   model } \item{ests.chain}{ a matrix with all of the coefficient estimates,
##'   at each EM iteration} \item{DM}{the DM matrix from the SEM algorithm}
##'   \item{DMiter}{a vector showing how many iterations it took for the
##'   variance component to converge in the SEM algorithm} }
##' @examples
##'     ## This is code from the CFR vignette provided in the documentation.
##'
##' data(simulated.outbreak.deaths)
##' min.cases <- 10
##' N.1 <- simulated.outbreak.deaths[1:60, "N"]
##' N.2 <- simulated.outbreak.deaths[61:120, "N"]
##' first.t <- min(which(N.1 > min.cases & N.2 > min.cases))
##' last.t <- max(which(N.1 > min.cases & N.2 > min.cases))
##' idx.for.Estep <- first.t:last.t
##' new.times <- 1:length(idx.for.Estep)
##' simulated.outbreak.deaths <- cbind(simulated.outbreak.deaths, new.times = NA)
##' simulated.outbreak.deaths[c(idx.for.Estep, idx.for.Estep + 60), "new.times"] <- rep(new.times, + 2)
##' assumed.nu = c(0, 0.3, 0.4, 0.3)
##' alpha.start <- rep(0, 22)
##'
##' ## caution! this next line may take several minutes (5-10, depanding on
##' ##    the speed of your machine) to run.
##' \dontrun{cfr.ests <- EMforCFR(assumed.nu = assumed.nu,
##'                               alpha.start.values = alpha.start,
##'                               full.data = simulated.outbreak.deaths,
##'                               verb = FALSE,
##'                               SEM.var = TRUE,
##'                               max.iter = 500,
##'                               tol = 1e-05)}
##' @keywords coarse data
##' @keywords incomplete data
##' @keywords case fatality ratio
##' @keywords infectious disease
##' @export
EMforCFR <- function(assumed.nu, alpha.start.values, full.data,
                     max.iter = 50, verb = FALSE, tol = 1e-10, SEM.var = TRUE) {
  ## full.data is the data output from the observe.epidemic() function
  ##    with a column specifying the rows to be used for analysis

  #######################################
  ## calculate naive and GLM estimates ##
  #######################################

  D.1 <- sum(full.data[full.data[, "grp"] == 1 & !is.na(full.data[, "new.times"]), "D"])
  D.2 <- sum(full.data[full.data[, "grp"] == 2 & !is.na(full.data[, "new.times"]), "D"])
  N.1 <- sum(full.data[full.data[, "grp"] == 1 & !is.na(full.data[, "new.times"]), "N"])
  N.2 <- sum(full.data[full.data[, "grp"] == 2 & !is.na(full.data[, "new.times"]), "N"])
  naive.rel.cfr <- sum(D.2) / sum(N.2) / (sum(D.1) / sum(N.1))

  dat <- data.frame(full.data)
  unadj.glm.fit <- glm(dat[, "D"] ~ factor(dat[, "new.times"]) + factor(dat[, "grp"]),
    offset = log(dat[, "N"]), family = poisson
  )
  glm.rel.cfr <- exp(coef(unadj.glm.fit))[length(coef(unadj.glm.fit))]

  #########################
  ## set up EM algorithm ##
  #########################
  proposed.rel.cfr.chain <- rep(0, max.iter)
  n.params <- max(dat[, "new.times"], na.rm = TRUE) + 1
  phi.chain <- matrix(nrow = n.params, ncol = max.iter)
  nlag <- length(assumed.nu)

  ## convergence codes:
  ## 0 = all parameters converge
  ## 1 = rCFR converges, but some other parameters do not
  ## 2 = rCFR does not converge
  EMconv <- SEMconv <- 0

  ## looping variables
  eps <- 1
  j <- 0
  alpha <- alpha.start.values
  while (eps > tol) {
    j <- j + 1
    ############
    ## E STEP ##
    ############
    dat <- run.Estep(alpha,
      full.data = full.data,
      nlag = nlag, assumed.nu = assumed.nu
    )

    ############
    ## M STEP ##
    ############
    maxLik <- run.Mstep(dat)
    phi.chain[, j] <- maxLik$phi
    alpha <- phi.chain[2:(n.params - 1), j]
    proposed.rel.cfr.chain[j] <- exp(phi.chain[n.params, j])

    ## check convergence
    if (j > 1) eps <- max((phi.chain[, j] - phi.chain[, j - 1])^2)
    if (j == max.iter) {
      ## message("*** WARNING: EM agorithm didn't converge ***")
      if ((phi.chain[n.params, j] - phi.chain[n.params, j - 1])^2 < eps) {
        EMconv <- 1
      } else {
        EMconv <- 2
      }
      break
    }
  }

  var.joint <- maxLik$Var

  phi <- phi.chain[, j]
  proposed.rel.cfr <- proposed.rel.cfr.chain[j]
  EMiter <- j
  if (verb) {
    print(paste("naive estimator =", round(naive.rel.cfr, 3)))
    print(paste("GLM estimate unadjusted for lags =", round(glm.rel.cfr, 3)))
    print(paste(EMiter, "iterations for EM convergence"))
  }

  if (SEM.var) {
    DM.out <- SEM.variance(full.data = full.data, dat, phi, max.iter, tol, nlag = nlag, alpha.start.values, assumed.nu = assumed.nu)
    DM <- DM.out$DM
    DMiter <- DM.out$DMiter
    if (length(DM.out$loop.idx) > 0) {
      loop.idx <- DM.out$loop.idx
      var.joint <- var.joint[-loop.idx, -loop.idx]
    }
    ## set SEM convergence codes
    if (any(DMiter == 0)) {
      if (DMiter[length(DMiter)] == 0) {
        SEMconv <- 2
      } else {
        SEMconv <- 1
      }
    }

    ## calculate CFR variance if it converged
    if (SEMconv < 2) {
      variance.SEM <- var.joint +
        var.joint %*% DM %*% solve(diag(1, ncol(DM)) - DM)
      proposed.rel.cfr.var.SEM <- variance.SEM[nrow(DM), nrow(DM)]
      if (proposed.rel.cfr.var.SEM < 0) {
        proposed.rel.cfr.var.SEM <- NA
        SEMconv <- 2
      }
    } else {
      proposed.rel.cfr.var.SEM <- NA
    }

    if (verb) {
      print("Estimate with SEM")
      print(paste(
        "proposed CFR =", round(proposed.rel.cfr, 3),
        "; 95% CI (",
        round(
          exp(log(proposed.rel.cfr) - 2 * sqrt(proposed.rel.cfr.var.SEM)),
          3
        ),
        ",",
        round(
          exp(log(proposed.rel.cfr) + 2 * sqrt(proposed.rel.cfr.var.SEM)),
          3
        ),
        ")"
      ))
      if (any(DM.out$DMiter == 0)) {
        print("non-convergent DM indices:")
        print(which(DM.out$DMiter == 0))
      }
    }
  } else {
    SEMconv <- NA
    DM <- NULL
    DMiter <- NULL
    proposed.rel.cfr.var.SEM <- NULL
  }

  ## ##############
  ## OUTPUT LIST ##
  ## ##############
  out <- list(
    naive.rel.cfr = naive.rel.cfr,
    glm.rel.cfr = glm.rel.cfr,
    EM.rel.cfr = proposed.rel.cfr,
    EM.rel.cfr.var = var.joint[nrow(var.joint), nrow(var.joint)],
    EM.rel.cfr.var.SEM = proposed.rel.cfr.var.SEM,
    EM.rel.cfr.chain = proposed.rel.cfr.chain,
    EMiter = EMiter,
    EMconv = EMconv,
    SEMconv = SEMconv,
    ests = phi,
    ests.chain.EM = phi.chain,
    DM = DM,
    DMiter = DMiter
  )

  if (verb) {
    print("Estimate with just GLM variance")
    print(paste(
      "proposed CFR =", round(proposed.rel.cfr, 3),
      "; 95% CI (",
      round(
        exp(log(proposed.rel.cfr) - 2 * sqrt(out$EM.rel.cfr.var)),
        3
      ),
      ",",
      round(
        exp(log(proposed.rel.cfr) + 2 * sqrt(out$EM.rel.cfr.var)),
        3
      ),
      ")"
    ))
  }

  return(out)
}


## This function is meant to be run only through the function EMforCFR()
##   and is used to calculate the variance via the Supplemented EM
##   algorithm (see Meng and Rubin, 1991)

## params:
##   \item{full.data}{ A matrix of observed data. See description in EMforCFR helpfile.}
##   \item{dat}{A data frame.}
##   \item{phi}{ A vector of fitted parameters from the final EM iteration.}
##   \item{max.iter}{ The maximum number of iterations for SEM algorithm. }
##   \item{tol}{ A tolerance to use to test for convergence of the EM algorithm. }
##   \item{nlag}{ The number of time units for lagged data.  Corresponds to length(assumed.nu).}
##   \item{alpha.start.values}{ a vector starting values for the reporting rate parameter of the GLM model. This must have length which corresponds to one less than the number of unique integer values of full.dat[,"new.times"].}
##   \item{assumed.nu}{ a vector of probabilities corresponding to the survival distribution, i.e. nu[i]=Pr(surviving i days | fatal case) }

## returned list:
##    \item{DM}{The estimate of the variance-covariance matrix for the model parameters.  Only converged rows are returned.}
##    \item{DMiter}{A vector whose ith entry is the number of iterations needed for convergence of the ith row of the DM matrix.}
##    \item{loop.idx}{If not NULL, the values correspond to the original indices of DM which have been omitted because of lack of convergence.}
SEM.variance <- function(full.data, dat, phi, max.iter, tol, nlag,
                         alpha.start.values, assumed.nu) {
  ## algorithm parameters
  iter <- 0
  phi.hat <- phi ## using phi from notation in Bayesian Data Analysis (Gelman, p323)
  phi0 <- c(1, alpha.start.values, 1) ## the starting values for the parameters: c(beta0, alpha2-T, gamma2)
  n.params <- length(phi0)

  ## sequence of matrices which will converge to DM
  Rt <- array(dim = c(n.params, n.params, max.iter))
  DM <- matrix(nrow = n.params, ncol = n.params)
  DMiter <- rep(0, n.params)

  ## sequence of coefficient estimates
  phi.t <- matrix(nrow = n.params, ncol = max.iter + 1)
  alpha.t <- alpha.start.values
  phi.t[, 1] <- phi0
  loop.idx <- 1:n.params
  while (length(loop.idx) > 0) {
    iter <- iter + 1

    ## THIS EM CALCULATES PHI^(T+1)
    ## E step
    dat <- run.Estep(alpha.t,
      nlag = nlag,
      full.data = full.data, assumed.nu = assumed.nu
    )
    ## M step
    phi.t[, iter + 1] <- run.Mstep(dat)$phi

    ## generate Rt given phi.t
    for (i in loop.idx) {
      ## defining phi.tmp here as phi^t(i) from Gelman
      phi.t.i <- phi.hat
      phi.t.i[i] <- phi.t[i, iter]
      alpha.t.i <- phi.t.i[2:(n.params - 1)]

      ## E step
      dat <- run.Estep(alpha.t.i,
        nlag = nlag,
        full.data = full.data, assumed.nu = assumed.nu
      )
      ## M step
      phi.tplus1.i <- run.Mstep(dat)$phi

      ## fis the ith row of the current Rt matrix
      Rt[i, , iter] <- (phi.tplus1.i - phi.hat) / (phi.t[i, iter] - phi.hat[i])
      if (iter == 1) next
      if (all((Rt[i, , iter] - Rt[i, , iter - 1])^2 < sqrt(tol))) {
        loop.idx <- loop.idx[-which(loop.idx == i)]
        DM[i, ] <- Rt[i, , iter]
        DMiter[i] <- iter
      }
    }

    ## renew alpha.t
    alpha.tplus1 <- phi.t[2:(n.params - 1), iter + 1]
    alpha.t <- alpha.tplus1

    if (iter == max.iter) break("maximum iterations reached")
  }

  if (length(loop.idx) > 0) DM <- DM[-loop.idx, -loop.idx]

  DM.out <- list(DM = DM, DMiter = DMiter, loop.idx = loop.idx)

  return(DM.out)
}


## E-step of the EM algorithm.

# \arguments{
#  \item{alpha}{Current estimates of the alpha parameters from the GLM model.}
#  \item{full.data}{ A matrix of observed data. See description in EMforCFR helpfile.}
#  \item{nlag}{ The number of time units for lagged data.  Corresponds to length(assumed.nu).}
#  \item{assumed.nu}{ a vector of probabilities corresponding to the survival distribution, i.e. nu[i]=Pr(surviving i days | fatal case) }
# }

# \value{ A data matrix with the same format as full.data from the EMforCFR() documentation.
# }

run.Estep <- function(alpha, full.data, nlag, assumed.nu) {
  ## storage for reconstructed deaths
  n.times <- nrow(full.data) / 2
  idx.to.reconstruct <- which(full.data[, "grp"] == 1 & !is.na(full.data[, "new.times"]))
  times.to.reconstruct <- full.data[idx.to.reconstruct, "time"]
  Dhat.1 <- Dhat.2 <- rep(0, n.times)

  ## define the data
  N.1 <- full.data[1:n.times, "N"]
  N.2 <- full.data[(n.times + 1):(2 * n.times), "N"]
  D.1 <- full.data[1:n.times, "D"]
  D.2 <- full.data[(n.times + 1):(2 * n.times), "D"]

  ## set the alpha coefficients
  alpha.long <- rep(0, n.times)
  alpha.long[times.to.reconstruct[-1]] <- alpha
  alpha.long[(max(times.to.reconstruct) + 1):n.times] <- alpha[length(alpha)]

  ## ########
  ## loops for calculating expected value of the deaths

  for (t in times.to.reconstruct) {
    denom.1 <- denom.2 <- rep(0, nlag)
    for (i in 1:nlag) {
      N.idx <- (t + i - 1):(t + i - nlag)
      nu.idx <- seq_along(N.idx)
      ## group 1
      denom.1[i] <- sum(assumed.nu[nu.idx] * N.1[N.idx] * exp(alpha.long[N.idx]))
      ## group 2
      denom.2[i] <- sum(assumed.nu[nu.idx] * N.2[N.idx] * exp(alpha.long[N.idx]))
    }

    ## exclude 0s from denominator
    denom.1[denom.1 == 0] <- NA
    denom.2[denom.2 == 0] <- NA

    ## sum them up
    D.idx <- (t + 1):(t + nlag)
    Dhat.1[t] <- N.1[t] * exp(alpha.long[t]) * sum(D.1[D.idx] * assumed.nu[nu.idx] / denom.1, na.rm = TRUE)
    Dhat.2[t] <- N.2[t] * exp(alpha.long[t]) * sum(D.2[D.idx] * assumed.nu[nu.idx] / denom.2, na.rm = TRUE)
  }

  dat <- cbind(
    Dhat = c(Dhat.1, Dhat.2),
    new.times = full.data[, "new.times"],
    grp = full.data[, "grp"],
    N = full.data[, "N"]
  )
  return(dat)
}


## the M-step

# \arguments{
#  \item{dat}{data matrix passed from EMforCFR().}
# }

# \value{ A list with two components
#         \item{phi }{fitted vector of parameters}
#         \item{Var }{variance-covariance matrix from the fitted model}
# }

run.Mstep <- function(dat) {
  subset <- which(dat[, "N"] > 0)
  fit <- glm(dat[, "Dhat"] ~ factor(dat[, "new.times"]) + factor(dat[, "grp"]),
    offset = log(dat[, "N"]), family = poisson, subset = subset
  )

  phi <- coef(fit)

  out <- list(phi = phi, Var = vcov(fit))
  return(out)
}

---
title: "Using outbreak data to estimate the relative case fatality ratio"
author: "Nicholas G Reich"
date: "`r format(Sys.time(), '%d %B %Y')`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Example for estimating the case fatality ratio}
  %\VignetteEngine{knitr::knitr}
  \usepackage[utf8]{inputenc}
---


## Introduction to the data

The goal of this vignette is to provide a tutorial for implementing the
case fatality ratio estimation methods available in the coarseDataTools R package.  To illustrate the methods, we will estimate a
relative case fatality ratio from a simulated outbreak dataset.


## Loading the data and the code
We begin by loading the package and a set of simulated data.

```{r label=sourcing}
library(coarseDataTools)
data(simulated.outbreak.deaths)
```

The data that we have loaded is a simulated dataset with five
columns.  Let's take a peek at the dataset.
```{r label=datapeek}
simulated.outbreak.deaths[15:20, ]
```
The rows index observations at a particular time and group.  The columns are defined as follows:

  -  time = the specified unit of time, $t$, for this observation
  -  grp = the covariate group, $j$, for this observation
  -  R = the number of recovered cases observed
  -  D = the number of dead cases observed
  -  N = the total number of cases observed, or D + R

We must perform a little bit of pre-processing on the data so that it
is ready for the analysis.  Specifically, we need to define the $T$
time periods that we want to use for analysis.  To avoid (for the
moment)  some convergence issues associated with
small sample sizes, we will only include times
where both groups have at least 10 total cases observed.

```{r label=preprocessing}
## set minimum number of observed cases for inclusion
min.cases <- 10

## observed cases
N.1 <- simulated.outbreak.deaths[1:60, "N"]
N.2 <- simulated.outbreak.deaths[61:120, "N"]

## subset to run analyis on times with greater than min.cases
first.t <- min(which(N.1 > min.cases & N.2 > min.cases))
last.t <- max(which(N.1 > min.cases & N.2 > min.cases))
idx.for.Estep <- first.t:last.t

## find and label the subset of times to be used for estimation routine
new.times <- seq_along(idx.for.Estep)
simulated.outbreak.deaths <- cbind(simulated.outbreak.deaths, new.times = NA)
simulated.outbreak.deaths[c(idx.for.Estep, idx.for.Estep + 60), "new.times"] <- rep(new.times, 2)
```

If we look at the new data matrix, we can see the new variable
that we've made and that will be used in the analysis:
```{r label=datapeek2}
simulated.outbreak.deaths[15:20, ]
```

## Running an analysis

We wish to fit a model to the simulated dataset that adjusts for changing reporting rates over time and
for a lag between disease onset and death.  We assume that the
reporting rates for dead and recovered cases vary by time but not by
covariate $j$.  We also assume that the case fatality ratio does not
vary by time but is different for each of the $j$ covariate groups.  The model formula that
adjusts for reporting rates but not the lag is
\begin{eqnarray}
\log E (D_{tj}) &=& \log N_{tj} + \beta_0 + \alpha_t\cdot X_t + \gamma_j\cdot Y_j
\end{eqnarray}
where $X_t$ and $Y_j$ are indicator variables  for $t=2,\dots,T$ and
$j=2, \dots,J$.  Then, $\gamma_j$ is the log relative case
fatality ratio.  The EM algorithm described in the main paper imputes
the values of $D_{tj}$ for the E-step and uses the
above equation as the M-step.

Before running an analysis, we need to fix a few parameters.
Specifically, we need to fix the assumed $\eta$, or the
vector of probabilities that define the survival distribution.  We
will assume that it is (0, .3, .4, .3).  For example, given that a
case will end up dying, there is a 40\% chance that the death will
occur on the third day after disease onset.  Also, we need to set
starting values for the $\alpha$ parameters in the model.  We will
assume that $\alpha_t=0$ for all $t$.

```{r label=setValues}
assumed.nu <- c(0, 0.3, 0.4, 0.3)
alpha.start <- rep(0, 22)
```

That is all the set up required.  We can now call the central
function, $\tt{EMforCFR()}$, which generates three estimates of the
CFR: the na\"ive, the reporting-rate-adjusted and the lag-adjusted estimators.
```{r label=runAnalysis, cache=TRUE, warning=FALSE}
cfr.ests <- EMforCFR(
  assumed.nu = assumed.nu,
  alpha.start.values = alpha.start, full.data = simulated.outbreak.deaths, verb = FALSE,
  SEM.var = TRUE, max.iter = 100, tol = 1e-5
)
```

[Note that running cfr.ests will generate warnings (many of them,
likely) but not to worry because this is due to the fact that the
likelihood uses non-integer outcomes in calculating the Poisson
density function.  This is an expected behavior and is not an
indication of a problem with the routine.]

The function $\tt{EMforCFR()}$ returns a list with the following components:

  -  naive.rel.cfr = the na\"ive estimator for the relative CFR
  -  glm.rel.cfr = the reporting-rate-adjusted estimator for the relative CFR
  -  EM.rel.cfr = the lag-adjusted estimator for the relative CFR
  -  EM.rel.cfr.var = the variance for the log-scale lag-adjusted
    estimator taken from the final M-step
  -  EM.rel.cfr.var.SEM = the Supplemented EM algorithm variance
    for the log-scale lag-adjusted estimator
  -  EM.rel.cfr.chain = a vector of the EM algorithm iterates of the
    lag-adjusted relative CFR estimates
  -  ests = the coefficient estimates for the model
  -  ests.chain.EM = a matrix with all of the coefficient
    estimates, at each EM iteration
  -  DM = the DM matrix from the SEM algorithm
  -  DMiter = a vector showing how many iterations it took for the
    variance component to converge in the SEM algorithm


We are particularly interested in the following components:
```{r label=estimationResults}
cfr.ests$naive.rel.cfr
cfr.ests$glm.rel.cfr
cfr.ests$EM.rel.cfr
cfr.ests$EM.rel.cfr.var.SEM
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/coarseDataTools-package.R
\docType{data}
\name{nycH1N1}
\alias{nycH1N1}
\title{Incubation period data from New York City Public Schools, 2009 H1N1 influenza outbreak}
\format{
A data frame with 134 observations on the following 5 variables.
\describe{
\item{\code{EL}}{the earliest possible time of infection}
\item{\code{ER}}{the latest possible time of infection}
\item{\code{SL}}{the earliest possible time of symptom onset}
\item{\code{SR}}{the latest possible time of symptom onset}
\item{\code{type}}{an indicator of the type of observation: 0 for doubly interval-censored, 1 for single-interval censored, 2 for exact. All of these observations are doubly interval-censored.}}
}
\source{
Lessler J, Reich NG, Cummings DAT and The DOHMH Swine Influenza Investigation Team. Outbreak of 2009 Pandemic Influenza A (H1N1) at a New York City School. New England Journal of Medicine. 2009. 361(27):2628-2636.
}
\usage{
data(nycH1N1)
}
\description{
These observations on the incubation period of influenza A come from the investigation of the H1N1 outbreak in NYC schools in the spring of 2009. They report doubly interval-censored observations for the incubation period.
}
\examples{
data(nycH1N1)
head(nycH1N1)
\dontrun{dic.fit(nycH1N1)}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/coarseDataTools-package.R
\docType{data}
\name{exp.win.lengths}
\alias{exp.win.lengths}
\title{Exposure window lengths from an influenza outbreak at a NYC school}
\format{
A numeric vector with 134 positive values.  Each value represents an exposure window length from an observation of the incubation period for that individual.  The exposure window length is the length of time during which exposure could have occurred.  For example, if an individual could have been exposed anytime between 6am on Monday to 6am on Wednesday, her exposure window length would be 2 days.
}
\source{
Lessler J et al.  New England Journal of Medicine. Outbreak of 2009 Pandemic Influenza A (H1N1) at a New York City School. 2009. 361(27):2628-2636. \doi{10.1056/NEJMoa0906089}
}
\usage{
data(exp.win.lengths)
}
\description{
A numeric vector of exposure window lengths taken from a dataset of doubly interval-censored incubation period observations.  All observations came from a NYC public school.  The outbreak has been described in full in Lessler et al. (see citation below).
}
\examples{
data(exp.win.lengths)
summary(exp.win.lengths)
hist(exp.win.lengths)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/S4stuff.R
\name{logLik,cd.fit-method}
\alias{logLik,cd.fit-method}
\alias{logLik}
\title{Get the log-likelihood value of a \code{cd.fit} or \code{cd.fit.mcmc} object}
\usage{
\S4method{logLik}{cd.fit}(object)
}
\arguments{
\item{object}{A \code{cd.fit} or \code{cd.fit.mcmc} object.}
}
\value{
log-likelihood value
}
\description{
Get the log-likelihood value of a \code{cd.fit} or \code{cd.fit.mcmc} object
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/coarseDataTools-package.R
\docType{data}
\name{fluA.inc.per}
\alias{fluA.inc.per}
\title{Coarse incubation period data for influenza A}
\format{
A data frame with 151 observations on the following 7 variables.
\describe{
\item{\code{author}}{the name of the primary author for the source of the observation}
\item{\code{year}}{the year of the study which is the source of the observation}
\item{\code{EL}}{the earliest possible time of infection}
\item{\code{ER}}{the latest possible time of infection}
\item{\code{SL}}{the earliest possible time of symptom onset}
\item{\code{SR}}{the latest possible time of symptom onset}
\item{\code{type}}{an indicator of the type of observation: 0 for doubly interval-censored, 1 for single-interval censored, 2 for exact}}
}
\source{
Lessler J, Reich NG, Brookmeyer R, Perl TM, Nelson KE, Cummings DAT. (2009) A systematic review of the incubation periods of acute respiratory viral infections. Lancet Infectious Diseases. 9(5):291-300.
}
\usage{
data(fluA.inc.per)
}
\description{
These observations on the incubation period of influenza A come from a variety of sources, and were gathered for a literature review.  They report doubly interval-censored, single interval-censored or exact observations for the incubation period.
}
\examples{
data(fluA.inc.per)
head(fluA.inc.per)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CFR_estimation.R
\name{EMforCFR}
\alias{EMforCFR}
\title{A function to estimate the relative case fatality ratio when reporting rates
are time-varying and deaths are lagged because of survival time.}
\usage{
EMforCFR(assumed.nu, alpha.start.values, full.data, max.iter = 50,
  verb = FALSE, tol = 1e-10, SEM.var = TRUE)
}
\arguments{
\item{assumed.nu}{a vector of probabilities corresponding to the survival
distribution, i.e. nu[i]=Pr(surviving i days | fatal case)}

\item{alpha.start.values}{a vector starting values for the reporting rate
parameter of the GLM model. This must have length which corresponds to one
less than the number of unique integer values of full.dat[,"new.times"].}

\item{full.data}{A matrix of observed data. See description below.}

\item{max.iter}{The maximum number of iterations for the EM algorithm and
the accompanying SEM algorithm (if used).}

\item{verb}{An indicator for whether the function should print results as it
runs.}

\item{tol}{A tolerance to use to test for convergence of the EM algorithm.}

\item{SEM.var}{If TRUE, the SEM algorithm will be run in addition to the EM
algorithm to calculate the variance of the parameter estimates.}
}
\value{
A list with the following elements \describe{ \item{naive.rel.cfr
  }{the naive estimate of the relative case fatality ratio}
  \item{glm.rel.cfr }{the reporting-rate-adjusted estimate of the relative
  case fatality ratio} \item{EM.rel.cfr }{the lag-adjusted estimate of the
  relative case fatality ratio} \item{EM.re.cfr.var }{the variance for the
  log-scale lag-adjusted estimator taken from the final M-step}
  \item{EM.rel.cfr.var.SEM }{ the Supplemented EM algorithm variance for the
  log-scale lag-adjusted estimator} \item{EM.rel.cfr.chain }{a vector of the
  EM algorithm iterates of the lag-adjusted relative CFR estimates}
  \item{EMiter}{the number of iterations needed for the EM algorithm to
  converge} \item{EMconv}{indicator for convergence of the EM algorithm.  0
  indicates all parameters converged within max.iter iterations.  1 indicates
  that the estimate of the relative case fatality ratio converged but other
  did not.  2 indicates that the relative case fatality ratio did not
  converge.} \item{SEMconv}{indicator for convergence of SEM algorithm.
  Same scheme as EMconv.} \item{ests}{ the coefficient estimates for the
  model } \item{ests.chain}{ a matrix with all of the coefficient estimates,
  at each EM iteration} \item{DM}{the DM matrix from the SEM algorithm}
  \item{DMiter}{a vector showing how many iterations it took for the
  variance component to converge in the SEM algorithm} }
}
\description{
This function implements an EM algorithm to estimate the relative case
fatality ratio between two groups when reporting rates are time-varying and
deaths are lagged because of survival time.
}
\details{
The data matrix full.data must have the following columns:
  \describe{ \item{grp}{a 1 or a 2 indicating which of the two groups, j,
  the observation is for.} \item{new.times}{an integer value representing
  the time, t, of observation.} \item{R}{the count of recovered cases with
  onset at time t in group j.} \item{D}{the count of deaths which occurred at
  time t in groupo j (note that these deaths did not have disease onset at
  time t but rather died at time t).} \item{N}{the total cases at t, j, or
  the sum of R and D columns.} }
}
\examples{
    ## This is code from the CFR vignette provided in the documentation.

data(simulated.outbreak.deaths)
min.cases <- 10
N.1 <- simulated.outbreak.deaths[1:60, "N"]
N.2 <- simulated.outbreak.deaths[61:120, "N"]
first.t <- min(which(N.1 > min.cases & N.2 > min.cases))
last.t <- max(which(N.1 > min.cases & N.2 > min.cases))
idx.for.Estep <- first.t:last.t
new.times <- 1:length(idx.for.Estep)
simulated.outbreak.deaths <- cbind(simulated.outbreak.deaths, new.times = NA)
simulated.outbreak.deaths[c(idx.for.Estep, idx.for.Estep + 60), "new.times"] <- rep(new.times, + 2)
assumed.nu = c(0, 0.3, 0.4, 0.3)
alpha.start <- rep(0, 22)

## caution! this next line may take several minutes (5-10, depanding on
##    the speed of your machine) to run.
\dontrun{cfr.ests <- EMforCFR(assumed.nu = assumed.nu,
                              alpha.start.values = alpha.start,
                              full.data = simulated.outbreak.deaths,
                              verb = FALSE,
                              SEM.var = TRUE,
                              max.iter = 500,
                              tol = 1e-05)}
}
\keyword{case}
\keyword{coarse}
\keyword{data}
\keyword{disease}
\keyword{fatality}
\keyword{incomplete}
\keyword{infectious}
\keyword{ratio}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dic.fit.R
\name{loglikhd}
\alias{loglikhd}
\title{Negative log likelihood for a dataset of interval-censored data, given a
distribution and its parameters.}
\usage{
loglikhd(pars, dat, dist)
}
\arguments{
\item{pars}{vector of the transformed (estimation scale) parameters}

\item{dat}{a dataset, as in \code{dic.fit}}

\item{dist}{a distribution, as in \code{dic.fit}}
}
\value{
negative log-likelihood for a given dataset, parameters, and
  distribution.
}
\description{
Negative log likelihood for a dataset of interval-censored data, given a
distribution and its parameters.
}
\details{
This package uses two versions of each parameter, the estimation
  scale, or the scale that is used for numerical optimization, and the
  reporting scale, or the natural scale of the parameters. For all
  likelihood calculations, this \code{loglikhd} function expects parameters
  that are on the estimation scale, i.e. have range \eqn{(-\infty, \infty)}.
  Specifically, this translates into all parameters for all distributions
  being log-transformed except for the meanlog (i.e. "par1") for the
  log-normal distribution.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dic.fit.R
\name{dgammaOff1}
\alias{dgammaOff1}
\title{Function that calculates dgamma with a offset of 1 (i.e., 1 is equivalent to 0)}
\usage{
dgammaOff1(x, replace0 = FALSE, ...)
}
\arguments{
\item{x}{value to calculate dgamma at}

\item{replace0}{should we replace 0 with epsilon}

\item{...}{other parameters to dgamma}
}
\value{
dgamma offset
}
\description{
Function that calculates dgamma with a offset of 1 (i.e., 1 is equivalent to 0)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sampleSizeSimulation.R
\name{precision.simulation}
\alias{precision.simulation}
\alias{precision.simulation.exact}
\alias{precision.simulation.coarse}
\alias{generate.coarse.data}
\title{Simulate incubation period analyses with coarse data}
\usage{
precision.simulation(
  N,
  med = 2,
  disp = 1.3,
  percentile = 0.5,
  nsim = 100,
  exact.data = FALSE,
  pct.type.A = 0.5,
  exp.win.dat = NULL,
  verb = FALSE
)

precision.simulation.exact(N, med, disp, percentile, nsim, verb)

precision.simulation.coarse(
  N,
  med,
  disp,
  percentile,
  nsim,
  pct.type.A,
  exp.win.dat,
  verb
)

generate.coarse.data(N, med, disp, pct.type.A, exp.win.dat)
}
\arguments{
\item{N}{Overall sample size for the datasets to be simulated.}

\item{med}{Median for the assumed log normal distribution of the incubation
periods.}

\item{disp}{Dispersion for the assumed log normal distribution of the
incubation periods.}

\item{percentile}{Percentile of the incubation period distribution which we
want to estimate.}

\item{nsim}{Number of datasets to analyze in the simulation.}

\item{exact.data}{Either TRUE/FALSE.  Incidates whether the data generated
should be coarsened at all.  If TRUE, pct.type.A and exp.win.dat are
ignored.}

\item{pct.type.A}{Percent of the N observations that are assumed to be type
A data.  If N*pct.type.A is not an integer, it will be rounded to the
nearest integer.}

\item{exp.win.dat}{A vector of exposure window lengths.  Defaults to the
observed window lengths from Lessler et al. (see below).}

\item{verb}{If TRUE, a message with the system time and iteration number
will be printed ten times during the simulation run.}
}
\value{
The \code{precision.simulation} functions return a matrix with four
  columns and nsim rows.  The "ests" column gives the estimated percentiles
  for the incubation period distribution.  The "SE" column gives the
  standard error for the estimate.  The "conv" column is 1 if the doubly
  interval-censored likelihood maximization converged.  Otherwise, it is 0.
  The "bias" column gives the estimated percentile - true percentile. The
  \code{generate.coarse.data} function returns a matrix with data suitable
  for analysis by the \code{dic.fit} function.
}
\description{
These functions simulate coarse incubation period data sets and
  analyze them.  The goal is for these simulations to provide evidence for
  how much information a given dataset contains about a characteristic of
  the incubation period distribution.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dic.fit.mcmc.R
\name{dic.fit.mcmc}
\alias{dic.fit.mcmc}
\title{Fits the distribution to the passed-in data using MCMC
as implemented in MCMCpack.}
\usage{
dic.fit.mcmc(
  dat,
  prior.par1 = NULL,
  prior.par2 = NULL,
  init.pars = c(1, 1),
  ptiles = c(0.05, 0.95, 0.99),
  verbose = 1000,
  burnin = 3000,
  n.samples = 5000,
  dist = "L",
  seed = NULL,
  ...
)
}
\arguments{
\item{dat}{the data}

\item{prior.par1}{vector of first prior parameters for each model parameter. If \code{NULL} then default parameters are used (as described in Details section).}

\item{prior.par2}{vector of second prior parameters for each model parameter. If \code{NULL} then default parameters are used (as described in Details section).}

\item{init.pars}{the initial parameter values (vector length = 2 )}

\item{ptiles}{returned percentiles of the survival survival distribution}

\item{verbose}{how often do you want a print out from MCMCpack on iteration number and M-H acceptance rate}

\item{burnin}{number of burnin samples}

\item{n.samples}{number of samples to draw from the posterior (after the burnin)}

\item{dist}{distribution to be used (L for log-normal,W for weibull, G for Gamma, and E for erlang, off1G for 1 day right shifted gamma)}

\item{seed}{seed for the random number generator for MCMC}

\item{...}{additional parameters to \link[MCMCpack]{MCMCmetrop1R}}
}
\value{
a cd.fit.mcmc S4 object
}
\description{
Similar to \code{\link{dic.fit}} but uses MCMC instead of a direct likelihood optimization routine to fit the model. Currently, four distributions are supported: log-normal, gamma, Weibull, and Erlang. See Details for prior specification.
}
\details{
The following models are used:
\deqn{Log-normal model: f(x) = \frac{1}{x*\sigma \sqrt{2 * \pi}} exp\{-\frac{(\log x - \mu)^2}{2 * \sigma^2}\}}
\deqn{Log-normal Default Prior: \mu ~ N(0, 1000), log(\sigma) ~ N(0,1000)}
\deqn{Weibull model: f(x) = \frac{\alpha}{\beta}(\frac{x}{\beta})^{\alpha-1} exp\{-(\frac{x}{\beta})^{\alpha}\}}
\deqn{Weibull Default Prior Specification: log(\alpha) ~ N( 0, 1000), \beta ~ Gamma(0.001,0.001)}
\deqn{Gamma model: f(x) = \frac{1}{\theta^k \Gamma(k)} x^{k-1} exp\{-\frac{x}{\theta}\}}

\deqn{Gamma Default Prior Specification: p(k,\theta) \propto \frac{1}{\theta} * \sqrt{k*TriGamma(k)-1}}
(Note: this is Jeffery's Prior when both parameters are unknown), and
\deqn{Trigamma(x) = \frac{\partial}{\partial x^2} ln(\Gamma(x))}.)
\deqn{Erlang model: f(x) = \frac{1}{\theta^k (k-1)!} x^{k-1} exp\{-\frac{x}{\theta}\}}
\deqn{Erlang Default Prior Specification: k \sim NBinom(100,1), log(\theta) \sim N(0,1000)}
(Note: parameters in the negative binomial distribution above represent mean and size, respectively)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dic.fit.mcmc.R
\name{mcmcpack.ll}
\alias{mcmcpack.ll}
\title{posterior log likelihood function to pass to MCMCpack sampler}
\usage{
mcmcpack.ll(pars, dat, prior.par1, prior.par2, dist)
}
\arguments{
\item{pars}{the parameters to calculate the ll at}

\item{dat}{the date to base it on}

\item{prior.par1}{first parameter of each prior}

\item{prior.par2}{second parameter of each prior}

\item{dist}{the distribution the likelihood is being calculated for}
}
\value{
the posterior log likelihood
}
\description{
posterior log likelihood function to pass to MCMCpack sampler
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dic.fit.mcmc.R
\name{mcmc.erlang}
\alias{mcmc.erlang}
\title{Does a metropolis hastings for the Erlang distribution}
\usage{
mcmc.erlang(
  dat,
  prior.par1,
  prior.par2,
  init.pars,
  verbose,
  burnin,
  n.samples,
  sds = c(1, 1)
)
}
\arguments{
\item{dat}{the data to fit}

\item{prior.par1}{mean of priors. A negative binomial (for shape) and a normal for log(scale)}

\item{prior.par2}{dispersion parameters for priors, dispersion for negative binomial, log scale sd for normal}

\item{init.pars}{the starting parameters on the reporting scale}

\item{verbose}{how often to print an update}

\item{burnin}{how many burnin iterations to do}

\item{n.samples}{the number of samples to keep and report back}

\item{sds}{the standard deviations for the proposal distribution}
}
\value{
a matrix of n.samples X 2 parameters, on the estimation scale
}
\description{
Does a metropolis hastings for the Erlang distribution
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/S4stuff.R
\docType{class}
\name{cd.fit}
\alias{cd.fit}
\alias{cd.fit-class}
\title{An S4 Class that stores a fitted coarse data object}
\description{
This is the output from \code{dic.fit()}, which contains the important bits of information about the model fit and key options used.
}
\section{Slots}{

 \describe{
   \item{\code{ests}:}{Matrix of class \code{"numeric"}. This matrix summarizes the results of fitting the model. Rows correspond to the first parameter , the second parameter and then percentiles specified by the ptiles argument. Columns correspond to the point estimate, the lower and upper bounds on the 95\% confidence interval and the standard error of the point estimate. If the maximization does not converge, this matrix is filled with NAs.}
   \item{\code{conv}:}{Object of class \code{"numeric"}. A value of 1 indicates successful convergence; 0 indicates unsuccessful convergence.}
   \item{\code{MSG}:}{Object of class \code{"character"}. The error message returned from \code{optim()} if the routine fails to converge.}
   \item{\code{loglik}:}{Object of class \code{"numeric"}. Value of the estimated maximum log-likelihood.}
   \item{\code{samples}:}{Object of class \code{"data.frame"}. Data frame of bootstrap estimates of parameters (if bootstraps were performed).}
   \item{\code{data}:}{Object of class \code{"data.frame"}. Original data used to fit model.}
   \item{\code{dist}:}{Object of class \code{"character"}. Failure time distribution fit to data. "L" for log-normal, "G" for gamma, "W" for Weibull, and "E" for Erlang.}
   \item{\code{inv.hessian}:}{Object of class \code{"matrix"}. The inverse of the hessian matrix for the likelihood surface at the MLE. Used to determine the standard errors for the percentiles. Note that optimization is done on a transformed scale with all parameters logged for all distributions except the first parameter of the log-normal distribution.}
   \item{\code{est.method}:}{Object of class \code{"character"}. Method used for estimation.}
   \item{\code{ci.method}:}{Object of class \code{"character"}. Method used for estimation of confidence/credible intervals.}
 }
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get.obs.type.R
\name{get.obs.type}
\alias{get.obs.type}
\title{Tries to guess the observation types (SIC, DIC, or exact).}
\usage{
get.obs.type(dat)
}
\arguments{
\item{dat}{a matrix of data, similar to what needs to be passed to \code{dic.fit()}.}
}
\value{
vector of guessed types
}
\description{
Tries to guess the observation types (SIC, DIC, or exact).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dic.fit.R
\name{dic.fit}
\alias{dic.fit}
\title{censored survival data}
\usage{
dic.fit(
  dat,
  start.par2 = log(2),
  opt.method = "L-BFGS-B",
  par1.int = log(c(0.5, 13)),
  par2.int = log(c(1.01, log(5))),
  ptiles = c(0.05, 0.95, 0.99),
  dist = "L",
  n.boots = 0,
  ...
)
}
\arguments{
\item{dat}{a matrix with columns named "EL", "ER", "SL", "SR", corresponding
to the left (L) and right (R) endpoints of the windows of possible
exposure (E) and symptom onset (S). Also, a "type" column must be
specified and have entries with 0, 1, or 2, corresponding to doubly
interval-censored, single interval-censored or exact observations,
respectively.}

\item{start.par2}{starting value for 2nd parameter of desired distribution}

\item{opt.method}{method used by optim}

\item{par1.int}{the log-scale interval of possible median values (in the
same units as the observations in dat).  Narrowing this interval can help
speed up convergence of the algorithm, but care must be taken so that
possible values are not excluded or that the maximization does not return
a value at an endpoint of this interval.}

\item{par2.int}{the log-scale interval of possible dispersion values}

\item{ptiles}{percentiles of interest}

\item{dist}{what distribution to use to fit the data. Default "L" for
log-normal. "G" for gamma, and "W" for Weibull.}

\item{n.boots}{number of bootstrap resamples (0 means that asymptotic results are desired)}

\item{...}{additional options passed to optim}
}
\value{
a cd.fit S4 object.
}
\description{
\code{dic.fit} fits a parametric accelerated failure time model to survival
data. It was developed with the application to estimating incubation periods of infectious diseases
in mind but is applicable to many general problems.
The data can be a mixture of doubly interval-censored, single
interval-censored or exact observations from a single univariate
distribution. Currently, three distributions are supported: log-normal,
gamma, and Weibull. (The Erlang distribution is supported in the
\code{dic.fit.mcmc} function, which implements an MCMC version of this
code.) We use a consistent (par1, par2) notation for each distribution, they
map in the following manner: \deqn{Log-normal(meanlog=par1, sdlog=par2)}
\deqn{Gamma(shape=par1, scale=par2)} \deqn{Weibull(shape=par1, scale=par2)}
Standard errors of parameters can be computed using closed-form asymptotic
formulae or using a bootstrap routine for log-normal and gamma models.
Currently, bootstrap SEs are the only option for the gamma models, which do
not have a closed form for the percentiles. \code{dic.fit()} calculates
asymptotic SEs by default, or whenever the \code{n.boots} option is set to
0. To compute bootstrap SEs, just set \code{n.boots} to be greater than
zero. \code{\link{dic.fit.mcmc}} also allows for Markov Chain Monte Carlo
fitting of these three parametric models and Erlang models as well.
}
\examples{
data(fluA.inc.per)
dic.fit(fluA.inc.per, dist="L")
}
\references{
Reich NG et al.  Statistics in Medicine.  Estimating incubation
  periods with coarse data. 2009.
  \url{https://pubmed.ncbi.nlm.nih.gov/19598148/}
}
\seealso{
\code{\link{cd.fit}}, \code{\link{dic.fit.mcmc}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/coarseDataTools-package.R
\docType{data}
\name{simulated.outbreak.deaths}
\alias{simulated.outbreak.deaths}
\title{Simulated case and death reports from a fictional outbreak}
\format{
\describe{
            \item{\code{time}}{time, t,  after start of outbreak}
            \item{\code{grp}}{an categorical variable indicating membership in one of two groups of a covariate, j}
            \item{\code{R}}{number of recovered cases reported at the given t and j}
            \item{\code{D}}{number of deaths reported at the given t and j}
            \item{\code{N}}{total number of cases and deaths reported at t and j, or D+R}
            }
}
\source{
Reich NG, Lessler J, Cummings DAT, Brookmeyer R. Estimating case fatality ratios from infectious disease surveillance data. Biometrics. 2012. 68(2): 598-606.
}
\usage{
data(simulated.outbreak.deaths)
}
\description{
This dataset provides reported counts of cases and deaths occurring at different time points across a simulated outbreak. Details of the data simulation algorithm are provided in the manuscript "Estimating case fatality ratios from infectious disease surveillance data" (Reich et al., under review, available upon request).
}
\examples{
data(simulated.outbreak.deaths)
head(simulated.outbreak.deaths)
plot(simulated.outbreak.deaths[simulated.outbreak.deaths[,"grp"]==1,"D"], type="l")
}
